## ---------------------------
##
## Run leave-out analysis
##
## ---------------------------

leaveout_re <- c("EggaN_SI", "EcoS_SI", "Juv_SI", "RussianS_SI", "EggaN_SI|EcoS_SI|Juv_SI|RussianS_SI", "aldist", "EggaN", "EggaS", "EcoS|Juv", "RussianS", "WinterS", "EcoS|Juv|RussianS|EggaS|WinterS", "EggaN|RussianS|EggaS|WinterS", "EcoS|Juv|EggaN|EggaS|WinterS") # A character vector specifying regular expressions to sequentially remove from likelihood components by setting their weight to 0. Do lapply(leaveout_re, function(k) grep(k, tmb_param$switch, value = TRUE)) %>% setNames(leaveout_re) to see which components the argument is about to leave out.

# if(Sys.info()["sysname"] == "Linux") {
#   library(flexiblas)
#   flexiblas_switch(flexiblas_load_backend("OPENBLAS-THREADS") )
# }

if(!dir.exists(file.path(base_dir, "leaveout"))) dir.create(file.path(base_dir, "leaveout"))

# init_lo_param <- tmb_param %>% 
#   mutate(value = ifelse(optimise, lower + parscale/2, value))

## Message

time_lo_start <- Sys.time()
message("Leave out started ", time_lo_start)
cat("Sequentially dropping")
lapply(leaveout_re, function(k) grep(k, init_lo_param$switch, value = TRUE)) %>% setNames(leaveout_re)

## Leave out parameter list

leaveout_pars <- lapply(leaveout_re, function(k) {
  init_lo_param %>%
    g3_init_guess(k, 0, NA, NA, 0) %>%
    mutate(value = ifelse(is.infinite(unlist(value)), 0, value))# %>%
    # filter(grepl("weight", switch)) # to check it went right
}) %>% setNames(leaveout_re)

save(leaveout_pars, file = file.path(base_dir, "leaveout", "leaveout_pars_in.Rdata"), compress = "xz")

# Optimisation ####

lo_param <- pbapply::pblapply(seq_along(leaveout_pars), function(i) {

  print(names(leaveout_pars[i]))
  out <- g3_optim(model = tmb_model,
                  params = leaveout_pars[[i]],
                  use_parscale = TRUE,
                  method = 'BFGS',
                  control = list(maxit = maxit #, reltol = 1e-9
                  ),
                  print_status = TRUE,
                  print_id = names(leaveout_pars)[i]
  )

  return(out)

} #, cl = ifelse(ncores > length(leaveout_pars), length(leaveout_pars), ncores)
) %>% setNames(leaveout_re)

# lo_param <- lapply(lo_param, function(k) if(inherits(k, "try-error")) NULL else k) # Clean crashes not to crash the entire script

time_lo_end <- Sys.time()
time_lo <- round(as.numeric(time_lo_end - time_lo_start, units = "mins"), 1)
message("Optimization finished ", time_lo_end, " after ", time_lo, " min")

## Write the times to a file
if(!file.exists(file.path(base_dir, "session/run_times.txt"))) {
  info_file <- file(file.path(base_dir, "session/run_times.txt"))
  close(info_file)
}

cat(
  c("Leave out:\n",
    "   started ", as.character(time_lo_start), "\n",
    "   finished ", as.character(time_lo_end), "\n",
    "   time ", time_lo, " min", "\n",
    "   element names: ", paste(names(sapply(lo_param, function(k) attributes(k)$summary$method)), collapse = ", "), "\n",
    "   method: ", paste(sapply(lo_param, function(k) attributes(k)$summary$method), collapse = ", "), "\n",
    "   convergence: ", paste(sapply(lo_param, function(k) attributes(k)$summary$convergence), collapse = ", "), "\n",
    "   iterations: ", paste(sapply(lo_param, function(k) attributes(k)$summary$gd_calls), collapse = ", "), "\n",
    "   score: ", paste(sapply(lo_param, function(k) {
      out <- attributes(k)$summary$score
      if(is.null(out)) NULL else round(out,1)
    })
    , collapse = ", "), "\n\n"),
  file = file.path(base_dir, "session/run_times.txt"), sep = "")

### Save the model parameters

save(lo_param, file = file.path(base_dir, "leaveout/Optimized leaveout parameters.rda"), compress = "xz")

## Fit

lo_fit <- pbapply::pblapply(seq_along(lo_param), function(i) {
  gadgetutils:::echo_message(paste0("Generating fit: ", i, "/", length(lo_param)))
  if(is.null(lo_param[[i]])) return(NULL)
  if(inherits(lo_param[[i]], "try-error")) return(NULL)
  try(g3_fit(model = tmb_model, params = lo_param[[i]]))
}#, cl = ifelse(ncores > length(leaveout_pars), length(leaveout_pars), ncores)
) %>% setNames(leaveout_re)

# lo_fit <- Filter(
#   Negate(is.null),
#   lapply(lo_fit, function(k) if(inherits(k, "try-error")) NULL else k)
# )

save(lo_fit, file = file.path(base_dir, "leaveout/Leaveout fit.rda"), compress = "xz")

## Plots

ggsave(file.path(base_dir, "figures/Leaveout_analysis.png"),
       plot = gadgetplots::plot_leaveout(lo_fit),
       width = pagewidth*1.4, height = pagewidth*0.7, units = "mm")

ggsave(
  file.path(base_dir, "figures/Leaveout_analysis_survey_indices.png"),
  plot =
    plot_leaveout(
      c(list("None" = optim_fit),
        lo_fit[c("EggaN_SI", "EcoS_SI", "Juv_SI", "RussianS_SI", 
                 "EggaN_SI|EcoS_SI|Juv_SI|RussianS_SI")] %>%
          setNames(c("EggaN_SI", "EcoS_SI", "Juv_SI", "RussianS_SI", "All"))
        ),
      colors = c("None" = "black", "EggaN_SI" = "#D696C8", "EcoS_SI" = "#056A89",
                 "Juv_SI" = "#449BCF", "RussianS_SI" = "#82C893", "All" = "grey60")
    ),
  width = pagewidth, height = pagewidth*0.7, units = "mm"
)

ggsave(
  file.path(base_dir, "figures/Leaveout_analysis_survey_data.png"),
  plot =
    plot_leaveout(
      c(list("None" = optim_fit),
        lo_fit[c("EggaN", "EggaS", "EcoS|Juv", "RussianS", "WinterS")]),
      colors = c("None" = "black", "EggaN" = "#D696C8", "EcoS|Juv" = "#056A89",
                 "RussianS" = "#82C893", "EggaS" = "#FF5F68", "WinterS" = "#FF9252")
    ),
  width = pagewidth, height = pagewidth*0.7, units = "mm"
)

ggsave(
  file.path(base_dir, "figures/Leaveout_analysis_age_data.png"),
  plot =
    plot_leaveout(
      c(list("None" = optim_fit),
        lo_fit[c("aldist")]),
      colors = c("None" = "black", "aldist" = "#D696C8")
    ),
  width = pagewidth, height = pagewidth*0.7, units = "mm"
)

ggsave(
  file.path(base_dir, "figures/Leavein_analysis_survey_data.png"),
  plot =
    plot_leaveout(
      c(list("All" = optim_fit),
        lo_fit[
          c("EcoS|Juv|RussianS|EggaS|WinterS",
            "EggaN|RussianS|EggaS|WinterS",
            "EcoS|Juv|EggaN|EggaS|WinterS")] %>%
          setNames(c("EggaN", "EcoS", "RussianS"))),
      colors = c("All" = "black", "EggaN" = "#D696C8", "EcoS" = "#056A89", "RussianS" = "#82C893"),
      legend_title = "Left-in\nsurvey"
    ),
  width = pagewidth, height = pagewidth*0.7, units = "mm"
)
