## ---------------------------
##
## Script name: Calculate TAC from short-term projection simulations using the harvest rate yielding maximum sustainable yield (HRmsy) from long-term projections, make reference point figures and tables.
##
## Purpose of script: Script to calculate TAC and other reference points. Source the entire script or run from top to a desired line (Ctrl+Alt+B on Rstudio).
##
## Instructions: Update data and optimize the model first using _RUN_THE_MODEL.R file. Once happy with the model, place all model files into a folder called latest_model in the model root (or change base_dir object which defines the folder). Once done, you can source this script and it should take care of the rest.
##
## Authors:
## Mikko Vihtakari and Daniel Howell // Institute of Marine Research, Norway
## Will Butler and Bjarki Elvarsson // Marine and Freshwater Research Institute, Iceland
## Email: mikko.vihtakari@hi.no
##
## ---------------------------

# rm(list = ls())

# Model settings ####
num_project_years <- 6 # How many years to project forward

last_advice <- c(`2024` = 15560) # pick from the advice sheets
last_tac <- c(`2024` = 21250)


## Set up folder structure ####
outpath <- file.path(base_dir, "short_term_projections") # Output folder for the projections
if(!dir.exists(outpath)) dir.create(outpath, recursive = TRUE)

# Model setup ####

source("R/projection_functions.R")

## Load data ####
load(file.path(base_dir, "optim", "Optimized TMB model fit.rda")) # Optimised model fit
fit <- optim_fit # In case there are still typos (otherwise fit will be taken from gadgetplots)
load(file.path(base_dir, "data", "Model and stock parameters.rda")) # Model setup
load(file.path(base_dir, "data", "Model stock actions.rda")) # Model stocks and stock actions
load(file.path(base_dir, "data", "TMB model.rda")) # The TMB model
load(file.path("data", 'reference_point_table.rda')) # Reference points from 2023
# load(file.path(base_dir, "long_term_projections", 'reference_point_table.rda')) # Reference points from the model

##################### ##
## Stock and fleets ####
##################### ##

hr_target <- rp_tab %>% filter(rp == "HR(target)msy") %>% pull(Value) # HR(target)msy from the RP table

year_range <- range(optim_fit$res.by.year$year)
end_year <- max(year_range)
start_year <- end_year + 1
num_steps <- length(unique(optim_fit$stock.full$step))

### Biological reference points ####
## Blim in kilograms
blim <- rp_tab %>% filter(rp == "Blim") %>% pull(Value) * 1e6 # from https://www.hi.no/hi/nettrapporter/imr-pinro-en-2023-6
# optim_fit$res.by.year %>%
# filter(year == 1992, stock == "ghl_female_mat") %>%
# pull(total.biomass)

bpa <- blim * 1.4 # Bpa following the ICES rule when no boostrapping for uncertainty.
btrigger <- bpa # Bpa = Btrigger

## Model schedule
recage <- stock_params$male_imm$minage
schedule <-
  expand.grid(year = (end_year - recage):(end_year + num_project_years),
              step = recstep)

### Recruitment values for bootstrapping ####
bootstr_rec <-
  fit$stock.recruitment %>%
  filter(recruitment > 0, year >= rec_start_year,
         year <= rec_end_year, step == recstep) %>%
  group_by(year) %>%
  summarise(recruitment = sum(recruitment))

### Stock actions ####
proj_actions0 <-
  c(attributes(tmb_model)$actions,
    proj_stock_actions2(num_project_years,
                        mat = female_mat,
                        imm_F = female_imm,
                        imm_M = male_imm))

##################################### ##
## Intermediate fleets and landings ####
##################################### ##

int_TrawlNor <- g3_fleet(c("TrawlNor", "int")) %>%
  g3s_livesonareas(areas[c('1')])
int_OtherNor <- g3_fleet(c("OtherNor", "int")) %>%
  g3s_livesonareas(areas[c('1')])
int_TrawlRus <- g3_fleet(c("TrawlRus", "int")) %>%
  g3s_livesonareas(areas[c('1')])
int_OtherRus <- g3_fleet(c("OtherRus", "int")) %>%
  g3s_livesonareas(areas[c('1')])
int_Internat <- g3_fleet(c("Internat", "int")) %>%
  g3s_livesonareas(areas[c('1')])

###################### ##
## Projection fleets ####
###################### ##

proj_TrawlNor <- g3_fleet(c("TrawlNor", "proj")) %>%
  g3s_livesonareas(areas[c('1')])
proj_OtherNor <- g3_fleet(c("OtherNor", "proj")) %>%
  g3s_livesonareas(areas[c('1')])
proj_TrawlRus <- g3_fleet(c("TrawlRus", "proj")) %>%
  g3s_livesonareas(areas[c('1')])
proj_OtherRus <- g3_fleet(c("OtherRus", "proj")) %>%
  g3s_livesonareas(areas[c('1')])
proj_Internat <- g3_fleet(c("Internat", "proj")) %>%
  g3s_livesonareas(areas[c('1')])

proj_effort_scalar <-
  structure(expand.grid(
    year = start_year:(start_year + num_project_years),
    step = 1,
    area = 1,
    scalar = 1),
    area_group = list(`1` = 1))

## WB comment - Effort scalar turned to zero for the intermediate year.
## This ensures the quota fleets don't catch anything in this year
proj_effort_scalar <-
  proj_effort_scalar %>%
  mutate(scalar = case_when(year == start_year ~ 0, TRUE ~ 1))

## Setup a time-varying parameter table for harvest rates
hr_vec <-
  gadget3:::f_substitute(
    ~g3_param_table('project_hr',
                    expand.grid(cur_year = seq(end_year + 1, end_year + py)),
                    ifmissing = 0),
    list(py = num_project_years))

exponentiate_fleets <- FALSE

################### ##
### Fleet actions ####
################### ##
# Copy from long-term projections

##### Projection fleet ####

proj_fleet_actions <-

  list(

    proj_TrawlNor %>%
      g3a_predate_fleet(
        stocks,
        suitabilities =
          stocks %>%
          set_names(.,map(.,'name')) %>%
          map(function(x)
            if(model_params$selectivity == "dome")
              g3_suitability_andersen(
                g3_parameterized('andersen.p0', by_stock = c('species','sex')),
                p1 = g3_parameterized('trawlnor.p1', by_stock = c('species', 'sex')),
                p2 = g3_parameterized('andersen.p2', by_stock = c('species', 'sex')),
                p3 = g3_parameterized('trawlnor.p3', by_stock = c('species', 'sex')),
                p4 = g3_parameterized('trawlnor.p4', by_stock = c('species', 'sex')),
                p5 = g3_parameterized('andersen.L', by_stock = c('species', 'sex'))
              )
            else if (model_params$selectivity == "s")
              g3_suitability_exponentiall50(
                g3_parameterized('trawlnor.alpha', by_stock = c('species', 'sex'),
                                 exponentiate = exponentiate_fleets),
                g3_parameterized('trawlnor.l50', by_stock = c('species', 'sex'),
                                 exponentiate = exponentiate_fleets)
              )
            else
              stop("selectivity '", model_params$selectivity, "' has not been implemented")
          ),
        catchability_f =
          g3experiments::g3a_predate_catchability_hockeyfleet(
            btrigger = g3_parameterized('btrigger', optimise = FALSE),
            hr = gadget3:::f_substitute(
              ~a*b, list(a = hr_vec, b = g3_parameterized('trawlnor_prop'))
            ),
            proportion_f = ~1,
            E = g3_timeareadata('TrawlNor_proj',
                                proj_effort_scalar,
                                value_field = 'scalar'),
            sum_stocks = list(female_mat)),
        run_f = ~cur_year_projection),

    proj_OtherNor %>%
      g3a_predate_fleet(
        stocks,
        suitabilities =
          stocks %>%
          set_names(.,map(.,'name')) %>%
          map(function(x)
            if(model_params$selectivity == "dome")
              g3_suitability_andersen(
                g3_parameterized('andersen.p0', by_stock = c('species', 'sex')),
                p1 = g3_parameterized('othernor.p1', by_stock = c('species', 'sex')),
                p2 = g3_parameterized('andersen.p2', by_stock = c('species', 'sex')),
                p3 = g3_parameterized('othernor.p3', by_stock = c('species', 'sex')),
                p4 = g3_parameterized('othernor.p4', by_stock = c('species', 'sex')),
                p5 = g3_parameterized('andersen.L', by_stock = c('species', 'sex'))
              )
            else if (model_params$selectivity == "s")
              g3_suitability_exponentiall50(
                g3_parameterized('othernor.alpha', by_stock = c('species', 'sex'),
                                 exponentiate = exponentiate_fleets),
                g3_parameterized('othernor.l50', by_stock = c('species', 'sex'),
                                 exponentiate = exponentiate_fleets)
              )
            else
              stop("selectivity '", model_params$selectivity, "' has not been implemented")
          ),
        catchability_f =
          g3experiments::g3a_predate_catchability_hockeyfleet(
            btrigger = g3_parameterized('btrigger', optimise = FALSE),
            hr = gadget3:::f_substitute(
              ~a*b, list(a = hr_vec, b = g3_parameterized('othernor_prop'))),
            proportion_f = ~1,
            E = g3_timeareadata('OtherNor_proj',
                                proj_effort_scalar,
                                value_field = 'scalar'),
            sum_stocks = list(female_mat)),
        run_f = ~cur_year_projection),

    proj_TrawlRus %>%
      g3a_predate_fleet(
        stocks,
        suitabilities =
          stocks %>%
          set_names(.,map(.,'name')) %>%
          map(function(x)
            if(model_params$selectivity == "dome")
              g3_suitability_andersen(
                g3_parameterized('andersen.p0', by_stock = c('species', 'sex')),
                p1 = g3_parameterized('trawlrus.p1', by_stock = c('species', 'sex')),
                p2 = g3_parameterized('andersen.p2', by_stock = c('species', 'sex')),
                p3 = g3_parameterized('trawlrus.p3', by_stock = c('species', 'sex')),
                p4 = g3_parameterized('trawlrus.p4', by_stock = c('species', 'sex')),
                p5 = g3_parameterized('andersen.L', by_stock = c('species', 'sex'))
              )
            else if (model_params$selectivity == "s")
              g3_suitability_exponentiall50(
                g3_parameterized('trawlrus.alpha', by_stock = c('species', 'sex'),
                                 exponentiate = exponentiate_fleets),
                g3_parameterized('trawlrus.l50', by_stock = c('species', 'sex'),
                                 exponentiate = exponentiate_fleets)
              )
            else
              stop("selectivity '", model_params$selectivity, "' has not been implemented")
          ),
        catchability_f =
          g3experiments::g3a_predate_catchability_hockeyfleet(
            btrigger = g3_parameterized('btrigger', optimise = FALSE),
            hr = gadget3:::f_substitute(
              ~a*b, list(a = hr_vec, b = g3_parameterized('trawlrus_prop'))),
            proportion_f = ~1,
            E = g3_timeareadata('TrawlRus_proj',
                                proj_effort_scalar,
                                value_field = 'scalar'),
            sum_stocks = list(female_mat)),
        run_f = ~cur_year_projection),

    proj_OtherRus %>%
      g3a_predate_fleet(
        stocks,
        suitabilities =
          stocks %>%
          set_names(.,map(.,'name')) %>%
          map(function(x)
            if(model_params$selectivity == "dome")
              g3_suitability_andersen(
                g3_parameterized('andersen.p0', by_stock = c('species', 'sex')),
                p1 = g3_parameterized('othernor.p1', by_stock = c('species', 'sex')),
                p2 = g3_parameterized('andersen.p2', by_stock = c('species', 'sex')),
                p3 = g3_parameterized('othernor.p3', by_stock = c('species', 'sex')),
                p4 = g3_parameterized('othernor.p4', by_stock = c('species', 'sex')),
                p5 = g3_parameterized('andersen.L', by_stock = c('species', 'sex'))
              )
            else if (model_params$selectivity == "s")
              g3_suitability_exponentiall50(
                g3_parameterized('othernor.alpha', by_stock = c('species', 'sex'),
                                 exponentiate = exponentiate_fleets),
                g3_parameterized('othernor.l50', by_stock = c('species', 'sex'),
                                 exponentiate = exponentiate_fleets)
              )
            else
              stop("selectivity '", model_params$selectivity, "' has not been implemented")
          ),
        catchability_f =
          g3experiments::g3a_predate_catchability_hockeyfleet(
            btrigger = g3_parameterized('btrigger', optimise = FALSE),
            hr = gadget3:::f_substitute(
              ~a*b, list(a = hr_vec, b = g3_parameterized('otherrus_prop'))),
            proportion_f = ~1,
            E = g3_timeareadata('OtherRus_proj',
                                proj_effort_scalar,
                                value_field = 'scalar'),
            sum_stocks = list(female_mat)),
        run_f = ~cur_year_projection),

    proj_Internat %>%
      g3a_predate_fleet(
        stocks,
        suitabilities =
          stocks %>%
          set_names(.,map(.,'name')) %>%
          map(function(x)
            if(model_params$selectivity == "dome")
              g3_suitability_andersen(
                g3_parameterized('andersen.p0', by_stock = c('species','sex')),
                p1 = g3_parameterized('trawlnor.p1', by_stock = c('species', 'sex')),
                p2 = g3_parameterized('andersen.p2', by_stock = c('species', 'sex')),
                p3 = g3_parameterized('trawlnor.p3', by_stock = c('species', 'sex')),
                p4 = g3_parameterized('trawlnor.p4', by_stock = c('species', 'sex')),
                p5 = g3_parameterized('andersen.L', by_stock = c('species', 'sex'))
              )
            else if (model_params$selectivity == "s")
              g3_suitability_exponentiall50(
                g3_parameterized('trawlnor.alpha', by_stock = c('species', 'sex'),
                                 exponentiate = exponentiate_fleets),
                g3_parameterized('trawlnor.l50', by_stock = c('species', 'sex'),
                                 exponentiate = exponentiate_fleets)
              )
            else
              stop("selectivity '", model_params$selectivity, "' has not been implemented")
          ),
        catchability_f =
          g3experiments::g3a_predate_catchability_hockeyfleet(
            btrigger = g3_parameterized('btrigger', optimise = FALSE),
            hr = gadget3:::f_substitute(
              ~a*b, list(a = hr_vec, b = g3_parameterized('internat_prop'))),
            proportion_f = ~1,
            E = g3_timeareadata('Internat_proj',
                                proj_effort_scalar,
                                value_field = 'scalar'),
            sum_stocks = list(female_mat)),
        run_f = ~cur_year_projection)
  )

##### Fixed TAC fleets ####

proj_fixed_fleet_actions <-

  list(

    g3_fleet(c("TrawlNor", "proj_fixed")) %>%
      g3s_livesonareas(areas[c('1')]) %>%
      g3a_predate_fleet(
        stocks,
        suitabilities =
          stocks %>%
          set_names(.,map(.,'name')) %>%
          map(function(x)
            if(model_params$selectivity == "dome")
              g3_suitability_andersen(
                g3_parameterized('andersen.p0', by_stock = c('species','sex')),
                p1 = g3_parameterized('trawlnor.p1', by_stock = c('species', 'sex')),
                p2 = g3_parameterized('andersen.p2', by_stock = c('species', 'sex')),
                p3 = g3_parameterized('trawlnor.p3', by_stock = c('species', 'sex')),
                p4 = g3_parameterized('trawlnor.p4', by_stock = c('species', 'sex')),
                p5 = g3_parameterized('andersen.L', by_stock = c('species', 'sex'))
              )
            else if (model_params$selectivity == "s")
              g3_suitability_exponentiall50(
                g3_parameterized('trawlnor.alpha', by_stock = c('species', 'sex'),
                                 exponentiate = exponentiate_fleets),
                g3_parameterized('trawlnor.l50', by_stock = c('species', 'sex'),
                                 exponentiate = exponentiate_fleets)
              )
            else
              stop("selectivity '", model_params$selectivity, "' has not been implemented")
          ),
        catchability_f =
          g3a_predate_catchability_totalfleet(
            E = gadget3:::f_substitute(
              ~a*b,
              list(a = g3_parameterized('fixed_TAC'),
                   b = g3_parameterized('trawlnor_prop'))
            )
          ),
        run_f = gadget3:::f_substitute(~cur_year > assessyear,
                                       list(assessyear = start_year))
      ),

    g3_fleet(c("OtherNor", "proj_fixed")) %>%
      g3s_livesonareas(areas[c('1')]) %>%
      g3a_predate_fleet(
        stocks,
        suitabilities =
          stocks %>%
          set_names(.,map(.,'name')) %>%
          map(function(x)
            if(model_params$selectivity == "dome")
              g3_suitability_andersen(
                g3_parameterized('andersen.p0', by_stock = c('species', 'sex')),
                p1 = g3_parameterized('othernor.p1', by_stock = c('species', 'sex')),
                p2 = g3_parameterized('andersen.p2', by_stock = c('species', 'sex')),
                p3 = g3_parameterized('othernor.p3', by_stock = c('species', 'sex')),
                p4 = g3_parameterized('othernor.p4', by_stock = c('species', 'sex')),
                p5 = g3_parameterized('andersen.L', by_stock = c('species', 'sex'))
              )
            else if (model_params$selectivity == "s")
              g3_suitability_exponentiall50(
                g3_parameterized('othernor.alpha', by_stock = c('species', 'sex'),
                                 exponentiate = exponentiate_fleets),
                g3_parameterized('othernor.l50', by_stock = c('species', 'sex'),
                                 exponentiate = exponentiate_fleets)
              )
            else
              stop("selectivity '", model_params$selectivity, "' has not been implemented")
          ),
        catchability_f =
          g3a_predate_catchability_totalfleet(
            E = gadget3:::f_substitute(
              ~a*b,
              list(a = g3_parameterized('fixed_TAC'),
                   b = g3_parameterized('othernor_prop'))
            )
          ),
        run_f = gadget3:::f_substitute(~cur_year > assessyear,
                                       list(assessyear = start_year))
      ),

    g3_fleet(c("TrawlRus", "proj_fixed")) %>%
      g3s_livesonareas(areas[c('1')]) %>%
      g3a_predate_fleet(
        stocks,
        suitabilities =
          stocks %>%
          set_names(.,map(.,'name')) %>%
          map(function(x)
            if(model_params$selectivity == "dome")
              g3_suitability_andersen(
                g3_parameterized('andersen.p0', by_stock = c('species', 'sex')),
                p1 = g3_parameterized('trawlrus.p1', by_stock = c('species', 'sex')),
                p2 = g3_parameterized('andersen.p2', by_stock = c('species', 'sex')),
                p3 = g3_parameterized('trawlrus.p3', by_stock = c('species', 'sex')),
                p4 = g3_parameterized('trawlrus.p4', by_stock = c('species', 'sex')),
                p5 = g3_parameterized('andersen.L', by_stock = c('species', 'sex'))
              )
            else if (model_params$selectivity == "s")
              g3_suitability_exponentiall50(
                g3_parameterized('trawlrus.alpha', by_stock = c('species', 'sex'),
                                 exponentiate = exponentiate_fleets),
                g3_parameterized('trawlrus.l50', by_stock = c('species', 'sex'),
                                 exponentiate = exponentiate_fleets)
              )
            else
              stop("selectivity '", model_params$selectivity, "' has not been implemented")
          ),
        catchability_f =
          g3a_predate_catchability_totalfleet(
            E = gadget3:::f_substitute(
              ~a*b,
              list(a = g3_parameterized('fixed_TAC'),
                   b = g3_parameterized('trawlrus_prop'))
            )
          ),
        run_f = gadget3:::f_substitute(~cur_year > assessyear,
                                       list(assessyear = start_year))
      ),

    g3_fleet(c("OtherRus", "proj_fixed")) %>%
      g3s_livesonareas(areas[c('1')]) %>%
      g3a_predate_fleet(
        stocks,
        suitabilities =
          stocks %>%
          set_names(.,map(.,'name')) %>%
          map(function(x)
            if(model_params$selectivity == "dome")
              g3_suitability_andersen(
                g3_parameterized('andersen.p0', by_stock = c('species', 'sex')),
                p1 = g3_parameterized('othernor.p1', by_stock = c('species', 'sex')),
                p2 = g3_parameterized('andersen.p2', by_stock = c('species', 'sex')),
                p3 = g3_parameterized('othernor.p3', by_stock = c('species', 'sex')),
                p4 = g3_parameterized('othernor.p4', by_stock = c('species', 'sex')),
                p5 = g3_parameterized('andersen.L', by_stock = c('species', 'sex'))
              )
            else if (model_params$selectivity == "s")
              g3_suitability_exponentiall50(
                g3_parameterized('othernor.alpha', by_stock = c('species', 'sex'),
                                 exponentiate = exponentiate_fleets),
                g3_parameterized('othernor.l50', by_stock = c('species', 'sex'),
                                 exponentiate = exponentiate_fleets)
              )
            else
              stop("selectivity '", model_params$selectivity, "' has not been implemented")
          ),
        catchability_f =
          g3a_predate_catchability_totalfleet(
            E = gadget3:::f_substitute(
              ~a*b,
              list(a = g3_parameterized('fixed_TAC'),
                   b = g3_parameterized('otherrus_prop'))
            )
          ),
        run_f = gadget3:::f_substitute(~cur_year > assessyear,
                                       list(assessyear = start_year))
      ),

    g3_fleet(c("Internat", "proj_fixed")) %>%
      g3s_livesonareas(areas[c('1')]) %>%
      g3a_predate_fleet(
        stocks,
        suitabilities =
          stocks %>%
          set_names(.,map(.,'name')) %>%
          map(function(x)
            if(model_params$selectivity == "dome")
              g3_suitability_andersen(
                g3_parameterized('andersen.p0', by_stock = c('species','sex')),
                p1 = g3_parameterized('trawlnor.p1', by_stock = c('species', 'sex')),
                p2 = g3_parameterized('andersen.p2', by_stock = c('species', 'sex')),
                p3 = g3_parameterized('trawlnor.p3', by_stock = c('species', 'sex')),
                p4 = g3_parameterized('trawlnor.p4', by_stock = c('species', 'sex')),
                p5 = g3_parameterized('andersen.L', by_stock = c('species', 'sex'))
              )
            else if (model_params$selectivity == "s")
              g3_suitability_exponentiall50(
                g3_parameterized('trawlnor.alpha', by_stock = c('species', 'sex'),
                                 exponentiate = exponentiate_fleets),
                g3_parameterized('trawlnor.l50', by_stock = c('species', 'sex'),
                                 exponentiate = exponentiate_fleets)
              )
            else
              stop("selectivity '", model_params$selectivity, "' has not been implemented")
          ),
        catchability_f =
          g3a_predate_catchability_totalfleet(
            E = gadget3:::f_substitute(
              ~a*b,
              list(a = g3_parameterized('fixed_TAC'),
                   b = g3_parameterized('internat_prop'))
            )
          ),
        run_f = gadget3:::f_substitute(~cur_year > assessyear,
                                       list(assessyear = start_year))
      )
  )

##### Intermediate fleets ####

int_fleet_actions <-

  list(

    int_TrawlNor %>%
      g3a_predate_fleet(
        stocks,
        suitabilities =
          stocks %>%
          set_names(.,map(.,'name')) %>%
          map(function(x)
            if(model_params$selectivity == "dome")
              g3_suitability_andersen(
                g3_parameterized('andersen.p0', by_stock = c('species','sex')),
                p1 = g3_parameterized('trawlnor.p1', by_stock = c('species', 'sex')),
                p2 = g3_parameterized('andersen.p2', by_stock = c('species', 'sex')),
                p3 = g3_parameterized('trawlnor.p3', by_stock = c('species', 'sex')),
                p4 = g3_parameterized('trawlnor.p4', by_stock = c('species', 'sex')),
                p5 = g3_parameterized('andersen.L', by_stock = c('species', 'sex'))
              )
            else if (model_params$selectivity == "s")
              g3_suitability_exponentiall50(
                g3_parameterized('trawlnor.alpha', by_stock = c('species', 'sex'),
                                 exponentiate = exponentiate_fleets),
                g3_parameterized('trawlnor.l50', by_stock = c('species', 'sex'),
                                 exponentiate = exponentiate_fleets)
              )
            else
              stop("selectivity '", model_params$selectivity, "' has not been implemented")
          ),
        catchability_f =
          # gadget3::g3a_predate_catchability_totalfleet(
          #   g3_timeareadata('TrawlNor_int',
          #                   expected_catch %>%
          #                     filter(fleet == 'TrawlNor_fishery') %>%
          #                     select(-fleet),
          #                   value_field = 'catch')),
          g3a_predate_catchability_totalfleet(
            E = gadget3:::f_substitute(
              ~a,
              list(a = g3_parameterized('trawlnor_int'))
            )
          ),
        run_f = gadget3:::f_substitute(~cur_year == assessyear,
                                       list(assessyear = start_year))
      ),

    int_OtherNor %>%
      g3a_predate_fleet(
        stocks,
        suitabilities =
          stocks %>%
          set_names(.,map(.,'name')) %>%
          map(function(x)
            if(model_params$selectivity == "dome")
              g3_suitability_andersen(
                g3_parameterized('andersen.p0', by_stock = c('species', 'sex')),
                p1 = g3_parameterized('othernor.p1', by_stock = c('species', 'sex')),
                p2 = g3_parameterized('andersen.p2', by_stock = c('species', 'sex')),
                p3 = g3_parameterized('othernor.p3', by_stock = c('species', 'sex')),
                p4 = g3_parameterized('othernor.p4', by_stock = c('species', 'sex')),
                p5 = g3_parameterized('andersen.L', by_stock = c('species', 'sex'))
              )
            else if (model_params$selectivity == "s")
              g3_suitability_exponentiall50(
                g3_parameterized('othernor.alpha', by_stock = c('species', 'sex'),
                                 exponentiate = exponentiate_fleets),
                g3_parameterized('othernor.l50', by_stock = c('species', 'sex'),
                                 exponentiate = exponentiate_fleets)
              )
            else
              stop("selectivity '", model_params$selectivity, "' has not been implemented")
          ),
        catchability_f =
          # gadget3::g3a_predate_catchability_totalfleet(
          #   g3_timeareadata('OtherNor_int',
          #                   expected_catch %>%
          #                     filter(fleet == 'OtherNor_fishery') %>%
          #                     select(-fleet),
          #                   value_field = 'catch')),
          g3a_predate_catchability_totalfleet(
            E = gadget3:::f_substitute(
              ~a,
              list(a = g3_parameterized('othernor_int'))
            )
          ),
        run_f = gadget3:::f_substitute(~cur_year == assessyear,
                                       list(assessyear = start_year))),

    int_TrawlRus %>%
      g3a_predate_fleet(
        stocks,
        suitabilities =
          stocks %>%
          set_names(.,map(.,'name')) %>%
          map(function(x)
            if(model_params$selectivity == "dome")
              g3_suitability_andersen(
                g3_parameterized('andersen.p0', by_stock = c('species', 'sex')),
                p1 = g3_parameterized('trawlrus.p1', by_stock = c('species', 'sex')),
                p2 = g3_parameterized('andersen.p2', by_stock = c('species', 'sex')),
                p3 = g3_parameterized('trawlrus.p3', by_stock = c('species', 'sex')),
                p4 = g3_parameterized('trawlrus.p4', by_stock = c('species', 'sex')),
                p5 = g3_parameterized('andersen.L', by_stock = c('species', 'sex'))
              )
            else if (model_params$selectivity == "s")
              g3_suitability_exponentiall50(
                g3_parameterized('trawlrus.alpha', by_stock = c('species', 'sex'),
                                 exponentiate = exponentiate_fleets),
                g3_parameterized('trawlrus.l50', by_stock = c('species', 'sex'),
                                 exponentiate = exponentiate_fleets)
              )
            else
              stop("selectivity '", model_params$selectivity, "' has not been implemented")
          ),
        catchability_f =
          # gadget3::g3a_predate_catchability_totalfleet(
          #   g3_timeareadata('TrawlRus_int',
          #                   expected_catch %>%
          #                     filter(fleet == 'TrawlRus_fishery') %>%
          #                     select(-fleet),
          #                   value_field = 'catch')),
          g3a_predate_catchability_totalfleet(
            E = gadget3:::f_substitute(
              ~a,
              list(a = g3_parameterized('trawlrus_int'))
            )
          ),
        run_f = gadget3:::f_substitute(~cur_year == assessyear,
                                       list(assessyear = start_year))),

    int_OtherRus %>%
      g3a_predate_fleet(
        stocks,
        suitabilities =
          stocks %>%
          set_names(.,map(.,'name')) %>%
          map(function(x)
            if(model_params$selectivity == "dome")
              g3_suitability_andersen(
                g3_parameterized('andersen.p0', by_stock = c('species', 'sex')),
                p1 = g3_parameterized('othernor.p1', by_stock = c('species', 'sex')),
                p2 = g3_parameterized('andersen.p2', by_stock = c('species', 'sex')),
                p3 = g3_parameterized('othernor.p3', by_stock = c('species', 'sex')),
                p4 = g3_parameterized('othernor.p4', by_stock = c('species', 'sex')),
                p5 = g3_parameterized('andersen.L', by_stock = c('species', 'sex'))
              )
            else if (model_params$selectivity == "s")
              g3_suitability_exponentiall50(
                g3_parameterized('othernor.alpha', by_stock = c('species', 'sex'),
                                 exponentiate = exponentiate_fleets),
                g3_parameterized('othernor.l50', by_stock = c('species', 'sex'),
                                 exponentiate = exponentiate_fleets)
              )
            else
              stop("selectivity '", model_params$selectivity, "' has not been implemented")
          ),
        catchability_f =
          # gadget3::g3a_predate_catchability_totalfleet(
          #   g3_timeareadata('OtherRus_int',
          #                   expected_catch %>%
          #                     filter(fleet == 'OtherRus_fishery') %>%
          #                     select(-fleet),
          #                   value_field = 'catch')),
          g3a_predate_catchability_totalfleet(
            E = gadget3:::f_substitute(
              ~a,
              list(a = g3_parameterized('otherrus_int'))
            )
          ),
        run_f = gadget3:::f_substitute(~cur_year == assessyear,
                                       list(assessyear = start_year))),

    int_Internat %>%
      g3a_predate_fleet(
        stocks,
        suitabilities =
          stocks %>%
          set_names(.,map(.,'name')) %>%
          map(function(x)
            if(model_params$selectivity == "dome")
              g3_suitability_andersen(
                g3_parameterized('andersen.p0', by_stock = c('species','sex')),
                p1 = g3_parameterized('trawlnor.p1', by_stock = c('species', 'sex')),
                p2 = g3_parameterized('andersen.p2', by_stock = c('species', 'sex')),
                p3 = g3_parameterized('trawlnor.p3', by_stock = c('species', 'sex')),
                p4 = g3_parameterized('trawlnor.p4', by_stock = c('species', 'sex')),
                p5 = g3_parameterized('andersen.L', by_stock = c('species', 'sex'))
              )
            else if (model_params$selectivity == "s")
              g3_suitability_exponentiall50(
                g3_parameterized('trawlnor.alpha', by_stock = c('species', 'sex'),
                                 exponentiate = exponentiate_fleets),
                g3_parameterized('trawlnor.l50', by_stock = c('species', 'sex'),
                                 exponentiate = exponentiate_fleets)
              )
            else
              stop("selectivity '", model_params$selectivity, "' has not been implemented")
          ),
        catchability_f =
          # gadget3::g3a_predate_catchability_totalfleet(
          #   g3_timeareadata('Internat_int',
          #                   expected_catch %>%
          #                     filter(fleet == 'Internat_fishery') %>%
          #                     select(-fleet),
          #                   value_field = 'catch')),
          g3a_predate_catchability_totalfleet(
            E = gadget3:::f_substitute(
              ~a,
              list(a = g3_parameterized('internat_int'))
            )
          ),
        run_f = gadget3:::f_substitute(~cur_year == assessyear,
                                       list(assessyear = start_year)))
  )

### Update actions ####

proj_actions <- c(proj_actions0, proj_fleet_actions, proj_fixed_fleet_actions,
                  int_fleet_actions)
proj_actions <- c(proj_actions,
                  list(gadget3::g3a_report_detail(proj_actions, run_f = ~TRUE)))

## Compile the TMB model and make parameter template ####

tmb_proj <- g3_to_tmb(proj_actions)
base.par.proj <- attributes(tmb_proj)$parameter_template

####################### ##
## Initial parameters ####

## Update values
base.par.proj$value[optim_fit$params$switch] <- optim_fit$params$value

## Other columns
base.par.proj <- base.par.proj %>%
  mutate(across(any_of(names(optim_fit$params)[-1:-4]),
                ~ coalesce(optim_fit$params[[cur_column()]][match(switch, optim_fit$params$switch)], .x)))

# New columns
# base.par.proj$value$project_years <- num_project_years
# base.par.proj$value$blim <- blim
# base.par.proj[base.par.proj$switch=="btrigger","optimise"] <- TRUE

base.par.proj <-
  base.par.proj %>%
  g3_init_guess('project_years', value = num_project_years) %>%
  g3_init_guess('blim', value = blim) %>%
  g3_init_guess('btrigger', value = btrigger) %>%
  g3_init_guess('project_hr', value = hr_target) %>%
  g3_init_guess('fixed_TAC', value = 0) %>%
  g3experiments::g3p_project_rec(
    recruitment = data.frame(recruitment = mean(bootstr_rec$recruitment)),
    method = 'constant') # %>%
# g3experiments::g3p_project_rec(
#   recruitment = data.frame(recruitment = mean(bootstr_rec$recruitment)/2),
#   rec_pattern = "ghl.rec.2018|ghl.rec.2022",
#   method = 'constant')

### Default intermediate catches ####
## Intermediate catches assume the catch from the previous year

expected_catch <-
  optim_fit$fleet.info %>%
  filter(year %in% end_year, !grepl('survey$', fleet)) %>%
  group_by(fleet) %>%
  summarise(catch = mean(amount), .groups = 'drop') %>%
  mutate(area = 1, year = start_year, step = 1)

## Year 2024 specialty: since TAC has been reduced expected catch must be reduced as well
## Catch set manually as TAC for 2024 + mean catch over TAC 2018-2023 (1800 t)

expected_catch <- expected_catch %>%
  mutate(prop = catch/sum(catch)) %>%
  mutate(catch = 23050*1e3*prop)

base.par.proj$value$trawlnor_int <- expected_catch %>%
  filter(fleet == 'TrawlNor_fishery') %>% pull(catch)
base.par.proj$value$othernor_int <- expected_catch %>%
  filter(fleet == 'OtherNor_fishery') %>% pull(catch)
base.par.proj$value$trawlrus_int <- expected_catch %>%
  filter(fleet == 'TrawlRus_fishery') %>% pull(catch)
base.par.proj$value$otherrus_int <- expected_catch %>%
  filter(fleet == 'OtherRus_fishery') %>% pull(catch)
base.par.proj$value$internat_int <- expected_catch %>%
  filter(fleet == 'Internat_fishery') %>% pull(catch)


### Fleet catch proportions ####

catch_props <- fit$fleet.info %>%
  dplyr::filter(amount > 1,
                year > max(model_params$year_range) - n_catch_years) %>%
  dplyr::mutate(fleet = gsub("_fishery", "", fleet)) %>%
  dplyr::group_by(fleet) %>%
  dplyr::summarise(amount = sum(amount)) %>%
  dplyr::ungroup() %>%
  dplyr::mutate(prop = amount/sum(amount))

base.par.proj$value$trawlnor_prop <- catch_props %>%
  filter(fleet == "TrawlNor") %>% pull(prop)
base.par.proj$value$othernor_prop <- catch_props %>%
  filter(fleet == "OtherNor") %>% pull(prop)
base.par.proj$value$trawlrus_prop <- catch_props %>%
  filter(fleet == "TrawlRus") %>% pull(prop)
base.par.proj$value$otherrus_prop <- catch_props %>%
  filter(fleet == "OtherRus") %>% pull(prop)
base.par.proj$value$internat_prop <- catch_props %>%
  filter(fleet == "Internat") %>% pull(prop)

# Compile the model ####

## Quicker to use the R model for prognosis as you are only doing a few runs

r_proj <- g3_to_r(proj_actions)

# test <-  g3_fit(
#   r_proj,
#   base.par.proj %>%
#     g3p_project_advice_error(
#       hr_target =  plot_hr(optim_fit, stocks = "ghl_female_mat", return_data = TRUE) %>%
#         filter(year == max(year)) %>% pull(value), advice_rho = 0, advice_cv = 0
#     ) %>%
#     g3_init_guess("btrigger", value = 1, optimise = 1)
# )

# Catch scenarios ####

### MSY with advice rule scenario ####

# proj_msy_fit <- g3_fit(r_proj, base.par.proj)

## Bug hack shit:

proj_msy_fit <- g3_fit(
  r_proj,
  base.par.proj %>%
    g3_init_guess('project_hr', value = c(0.18, 0.181, 0.19, 0.194, 0.193, 0.188))
)

# plot_catch(proj_msy_fit, return_data = TRUE) %>%
#   group_by(year) %>%
#   summarise(Catches = round(sum(value*1e3, na.rm = TRUE), 0)) %>%
#   ungroup() %>% tail(10)

# SSB
# plot_biomass(proj_msy_fit, stocks = "ghl_female_mat", return_data = TRUE) %>%
#   dplyr::select(year, value) %>%
#   tail()

### MSY without advice rule scenario ####

# proj_msy_nobtrigger_fit <- g3_fit(
#   r_proj,
#   base.par.proj %>%
#     g3_init_guess('btrigger', value = 1)
# )

## Bug hack shit:

proj_msy_nobtrigger_fit <- g3_fit(
  r_proj,
  base.par.proj %>%
    g3_init_guess('project_hr', value = c(0.18, 0.181, 0.19, 0.194, 0.193, 0.188)) %>%
    g3_init_guess('btrigger', value = 1)
)

# plot_hr(proj_msy_nobtrigger_fit, min_catch_length = 45, return_data = TRUE) %>% tail
# plot_hr(proj_msy_nobtrigger_fit, min_catch_length = 0, return_data = TRUE) %>% tail

# plot_catch(proj_msy_nobtrigger_fit, return_data = TRUE) %>%
#   group_by(year) %>%
#   summarise(Catches = round(sum(value*1e3, na.rm = TRUE), 0)) %>%
#   ungroup() %>% tail()

### MSY with 2019 recruitment ignored ###

# proj_msy_reccut_fit <- g3_fit(
#   r_proj,
#   base.par.proj %>%
#     g3experiments::g3p_project_rec(
#       recruitment = data.frame(recruitment = mean(bootstr_rec$recruitment)/2),
#       rec_pattern = "ghl.rec.2018|ghl.rec.2022",
#       method = 'constant')
# )

### No catches scenario ####

proj_hr0_fit <- g3_fit(
  r_proj,
  base.par.proj %>% g3_init_guess('project_hr', value = 0))

# Manual HR hack code: ###
#
# proj_test_fit <- g3_fit(
#   r_proj,
#   base.par.proj %>%
#     g3_init_guess('project_hr', value = c(0.18, 0.181, 0.19, 0.194, 0.193, 0.188)) %>%
#     g3_init_guess('btrigger', value = 1)
# )
#
# plot_hr(proj_test_fit) + geom_hline(yintercept = 0.18)
# plot_hr(proj_test_fit, min_catch_length = 0) + geom_hline(yintercept = hr_target)
# plot_hr(proj_test_fit, min_catch_length = 45) + geom_hline(yintercept = 0.139)
# plot_hr(proj_test_fit, min_catch_length = 0, return_data = TRUE) %>% tail
# plot_hr(proj_test_fit, min_catch_length = 45, return_data = TRUE) %>% tail
# plot_hr(proj_test_fit, stock = "ghl_female_mat", return_data = TRUE) %>% tail

### Status quo catch scenario ####

proj_hrly_fit <- g3_fit(
  r_proj,
  base.par.proj %>%
    g3_init_guess('project_hr', value = 0) %>%
    g3_init_guess('fixed_TAC', value = sum(expected_catch$catch))
)

# plot_catch(proj_hrly_fit, return_data = TRUE) %>%
#   group_by(year) %>%
#   summarise(Catches = round(sum(value*1e3, na.rm = TRUE), 0)) %>%
#   ungroup() %>% tail

### Status quo harvest rate scenario ###

# tmp_int_catch <- 24.2*1e6
#
# proj_hrly_fit <- g3_fit(
#   r_proj,
#   base.par.proj %>%
#     g3_init_guess(
#       "trawlnor_int",
#       value = catch_props[catch_props$fleet == "TrawlNor",]$prop*tmp_int_catch
#     ) %>%
#     g3_init_guess(
#       "othernor_int",
#       value = catch_props[catch_props$fleet == "OtherNor",]$prop*tmp_int_catch
#     ) %>%
#     g3_init_guess(
#       "trawlrus_int",
#       value = catch_props[catch_props$fleet == "TrawlRus",]$prop*tmp_int_catch
#     ) %>%
#     g3_init_guess(
#       "otherrus_int",
#       value = catch_props[catch_props$fleet == "OtherRus",]$prop*tmp_int_catch
#     ) %>%
#     g3_init_guess(
#       "internat_int",
#       value = catch_props[catch_props$fleet == "Internat",]$prop*tmp_int_catch
#     )
# )

## Adjust the tmp_int_catch above to get same HR for 2024 than 2023

# plot_hr(proj_hrly_fit, min_catch_length = 0, return_data = T) %>% tail(n=10)

### 23 kt intermediate catch scenario under MSY approach ###

# tmp_int_catch <- 23050*1e3
#
# proj_tac_fit <- g3_fit(
#   r_proj,
#   base.par.proj %>%
#     g3_init_guess(
#       "trawlnor_int",
#       value = catch_props[catch_props$fleet == "TrawlNor",]$prop*tmp_int_catch
#       ) %>%
#     g3_init_guess(
#       "othernor_int",
#       value = catch_props[catch_props$fleet == "OtherNor",]$prop*tmp_int_catch
#     ) %>%
#     g3_init_guess(
#       "trawlrus_int",
#       value = catch_props[catch_props$fleet == "TrawlRus",]$prop*tmp_int_catch
#     ) %>%
#     g3_init_guess(
#       "otherrus_int",
#       value = catch_props[catch_props$fleet == "OtherRus",]$prop*tmp_int_catch
#     ) %>%
#     g3_init_guess(
#       "internat_int",
#       value = catch_props[catch_props$fleet == "Internat",]$prop*tmp_int_catch
#     )
# )

### NFD 25 kt catch scenario ###
#
# proj_25kt_fit <- g3_fit(
#   r_proj,
#   base.par.proj %>%
#     g3experiments::g3p_project_rec(
#       recruitment = data.frame(recruitment = mean(bootstr_rec$recruitment)/2),
#       rec_pattern = "ghl.rec.2018|ghl.rec.2022",
#       method = 'constant') %>%
#     g3_init_guess(
#       'project_hr',
#       value = c(0.30, 0.293, 0.329, 0.375, 0.434, 0.499)
#     ) %>%
#     g3_init_guess('btrigger', value = 1)
# )
#
# # Adjust the hr above to get 25k catches for next_year
# plot_catch(proj_25kt_fit, return_data = TRUE) %>%
#   group_by(year) %>%
#   summarise(Catches = round(sum(value*1e3, na.rm = TRUE), 0)) %>%
#   ungroup() %>% tail()
#
# # SSB
# plot_biomass(proj_25kt_fit, stocks = "ghl_female_mat", return_data = TRUE) %>%
#   dplyr::select(year, value) %>%
#   tail()
#
# ### NFD 30 kt catch scenario ###
#
# proj_30kt_fit <- g3_fit(
#   r_proj,
#   base.par.proj %>%
#     g3experiments::g3p_project_rec(
#       recruitment = data.frame(recruitment = mean(bootstr_rec$recruitment)/2),
#       rec_pattern = "ghl.rec.2018|ghl.rec.2022",
#       method = 'constant') %>%
#     g3_init_guess(
#       'project_hr',
#       value = c(0.36, 0.351, 0.417, 0.513, 0.659, 0.875)
#     ) %>%
#     g3_init_guess('btrigger', value = 1)
# )
#
# # Adjust the hr above to get 30k catches for next_year
# plot_catch(proj_30kt_fit, return_data = TRUE) %>%
#   group_by(year) %>%
#   summarise(Catches = round(sum(value*1e3, na.rm = TRUE), 0)) %>%
#   ungroup() %>% tail()
#
# # SSB
# plot_biomass(proj_30kt_fit, stocks = "ghl_female_mat", return_data = TRUE) %>%
#   dplyr::select(year, value) %>%
#   tail()
#
# ### NFD Blim by 2026 catch scenario ###
#
# proj_2026Blim_fit <- g3_fit(
#   r_proj,
#   base.par.proj %>%
#     g3experiments::g3p_project_rec(
#       recruitment = data.frame(recruitment = mean(bootstr_rec$recruitment)/2),
#       rec_pattern = "ghl.rec.2018|ghl.rec.2022",
#       method = 'constant') %>%
#     g3_init_guess(
#       'project_hr',
#       value = c(1,  0.293, 0.3, 0.375, 0.434, 0.499)
#       # value = c(1, 0.285, 0.315, 0.375, 0.434, 0.499)
#     ) %>%
#     g3_init_guess('btrigger', value = 1)
# )
#
# # Adjust the hr above to get 25k catches for next_year
# plot_catch(proj_2026Blim_fit, return_data = TRUE) %>%
#   group_by(year) %>%
#   summarise(Catches = round(sum(value*1e3, na.rm = TRUE), 0)) %>%
#   ungroup() %>% tail()
#
# # SSB
# plot_biomass(proj_2026Blim_fit, stocks = "ghl_female_mat", return_data = TRUE) %>%
#   dplyr::select(year, value) %>%
#   tail()
#
# ### NFD Blim by 2028 catch scenario ###
#
# proj_2028Blim_fit <- g3_fit(
#   r_proj,
#   base.par.proj %>%
#     g3experiments::g3p_project_rec(
#       recruitment = data.frame(recruitment = mean(bootstr_rec$recruitment)/2),
#       rec_pattern = "ghl.rec.2018|ghl.rec.2022",
#       method = 'constant') %>%
#     g3_init_guess(
#       'project_hr',
#       # value = c(0, 0.22, 0.23, 0.243, 0.253, 0.263)
#       value = c(0, 0.293, 0.2155, 0.225, 0.2285, 0.2345)
#     ) %>%
#     g3_init_guess('btrigger', value = 1)
# )
#
# # Adjust the hr above to get 25k catches for next_year
# plot_catch(proj_2028Blim_fit, return_data = TRUE) %>%
#   group_by(year) %>%
#   summarise(Catches = round(sum(value*1e3, na.rm = TRUE), 0)) %>%
#   ungroup() %>% tail()
#
# # SSB
# plot_biomass(proj_2028Blim_fit, stocks = "ghl_female_mat", return_data = TRUE) %>%
#   dplyr::select(year, value) %>%
#   tail()
#
## ###

# tmppath <- file.path(getwd(), base_dir, "figures")
# make_html(test, path = tmppath, file_name = "test.html", template = "nea_ghl")

# Figures ####

min_year = 2024

### HR plot ####

ggsave(
  plot =
    print(
      bind_rows(
        plot_hr(proj_msy_fit, stocks = "ghl_female_mat", return_data = TRUE) %>%
          mutate(type = "MSY") %>% dplyr::select(type, year, value),
        plot_hr(proj_msy_nobtrigger_fit, stocks = "ghl_female_mat", return_data = TRUE) %>%
          mutate(type = "No Btrigger") %>% dplyr::select(type, year, value),
        plot_hr(proj_hr0_fit, stocks = "ghl_female_mat", return_data = TRUE) %>%
          mutate(type = "HR0") %>% dplyr::select(type, year, value),
        plot_hr(proj_hrly_fit, stocks = "ghl_female_mat", return_data = TRUE) %>%
          mutate(type = paste("Assumed", start_year, "catch")) %>%
          dplyr::select(type, year, value)
      ) %>%
        ggplot(aes(x = year, y = value, color = type)) +
        geom_hline(yintercept = hr_target, color = "grey", linetype = 2) +
        geom_line() +
        labs(y = "Internal harvest rate (HR for female SSB)", x = "Year", color = "Scenario") +
        ggplot2::scale_x_continuous(breaks = seq(1980,2030,1)) +
        coord_cartesian(expand = FALSE, xlim = c(min_year, NA)) +
        ggplot2::theme_classic(base_size = 8) +
        theme(legend.position = "bottom",
              strip.background = element_blank(),
              panel.background = element_blank(),
              plot.background = element_blank(),
              legend.background = element_blank(),
              legend.box.background = element_blank(),
              plot.margin = margin(c(5.5, 10, 5.5, 5.5))) %>%
        suppressWarnings()
    ),
  filename = file.path(base_dir, "figures/Short-term_projection_HRs.png"),
  units = 'in', width = pagewidth_in, height = pagewidth_in*0.6,
  bg = "white")

ggsave(
  plot =
    print(
      bind_rows(
        plot_hr(proj_msy_fit, min_catch_length = 45, return_data = TRUE) %>%
          mutate(type = "MSY") %>% dplyr::select(type, year, value),
        plot_hr(proj_msy_nobtrigger_fit, min_catch_length = 45, return_data = TRUE) %>%
          mutate(type = "No Btrigger") %>% dplyr::select(type, year, value),
        plot_hr(proj_hr0_fit, min_catch_length = 45, return_data = TRUE) %>%
          mutate(type = "HR0") %>% dplyr::select(type, year, value),
        plot_hr(proj_hrly_fit, min_catch_length = 45, return_data = TRUE) %>%
          mutate(type = paste("Assumed", start_year, "catch")) %>%
          dplyr::select(type, year, value)
      ) %>%
        ggplot(aes(x = year, y = value, color = type)) +
        geom_hline(
          yintercept = rp_tab %>% filter(rp == "HR(bar)msy") %>% pull(Value),
          color = "grey", linetype = 2) +
        geom_line() +
        labs(y = expression("Harvest rate ">="45 cm"), x = "Year",
             color = "Scenario") +
        ggplot2::scale_x_continuous(breaks = seq(1980,2030,1)) +
        coord_cartesian(expand = FALSE, xlim = c(min_year, NA)) +
        ggplot2::theme_classic(base_size = 8) +
        theme(legend.position = "bottom",
              strip.background = element_blank(),
              panel.background = element_blank(),
              plot.background = element_blank(),
              legend.background = element_blank(),
              legend.box.background = element_blank(),
              plot.margin = margin(c(5.5, 10, 5.5, 5.5))) %>%
        suppressWarnings()
    ),
  filename = file.path(base_dir, "figures/Short-term_projection_HR45s.png"),
  units = 'in', width = pagewidth_in, height = pagewidth_in*0.6,
  bg = "white")

# ggsave(
#   plot =
#     print(
#       bind_rows(
#         plot_hr(proj_msy_fit, stocks = "ghl_female_mat", return_data = TRUE) %>%
#           mutate(type = "MSY") %>% dplyr::select(type, year, value),
#         plot_hr(proj_25kt_fit, stocks = "ghl_female_mat", return_data = TRUE) %>%
#           mutate(type = "25 kt") %>% dplyr::select(type, year, value),
#         plot_hr(proj_30kt_fit, stocks = "ghl_female_mat", return_data = TRUE) %>%
#           mutate(type = "30 kt") %>% dplyr::select(type, year, value)
#       ) %>%
#         ggplot(aes(x = year, y = value, color = type)) +
#         geom_line() +
#         labs(y = "Internal harvest rate (HR for female SSB)", x = "Year", color = "Scenario") +
#         coord_cartesian(expand = FALSE) +
#         ggplot2::theme_classic(base_size = 8) +
#         theme(legend.position = "bottom")
#     ),
#   filename = file.path(base_dir, "figures/NFD_projection_HRs.png"),
#   units = 'in', width = pagewidth_in, height = pagewidth_in*0.6,
#   bg = "white")

### Short-term projection plot ####

rp_cols <- c("MSY" = "#6CA67A", "HRmsy" = "#82C893", "Bpa" = "#056A89", "HRpa" = "#449BCF", "Blim" = "#D44F56", "HRlim" = "#FF5F68")

line_types <- c("MSY" = 1, "0" = 2, "No Btrigger" = 3, 4)
names(line_types)[4] <- paste("Assumed", start_year, "catch")

p <-
  bind_rows(
    plot_biomass(proj_hr0_fit, stocks = "ghl_female_mat", return_data = TRUE) %>%
      dplyr::select(year, value) %>%
      # filter(year <= start_year+3) %>%
      mutate(HR = "0"),
    plot_biomass(proj_msy_fit, stocks = "ghl_female_mat", return_data = TRUE) %>%
      dplyr::select(year, value) %>%
      # filter(year <= start_year+3) %>%
      mutate(HR = "MSY"),
    plot_biomass(proj_msy_nobtrigger_fit, stocks = "ghl_female_mat", return_data = TRUE) %>%
      dplyr::select(year, value) %>%
      # filter(year <= start_year+3) %>%
      mutate(HR = "No Btrigger"),
    plot_biomass(proj_hrly_fit, stocks = "ghl_female_mat", return_data = TRUE) %>%
      dplyr::select(year, value) %>%
      # filter(year <= start_year+3) %>%
      mutate(HR = paste("Assumed", start_year, "catch"))
  ) %>%
  ggplot(ggplot2::aes(.data$year, .data$value, linetype = .data$HR)) +
  ggplot2::geom_line() +
  geom_hline(data = rp_tab %>%
               filter(rp %in% c("Bpa", "Blim")),
             aes(yintercept = Value, color = rp)) +
  ggplot2::expand_limits(y = 0) +
  coord_cartesian(expand = FALSE, xlim = c(min_year, NA)) +
  ggplot2::scale_x_continuous(breaks = seq(1980,2030,1)) +
  scale_linetype_manual(values = line_types) +
  labs(x = "Year", y = "Female spawning stock biomass (kt)", color =
         "Reference\npoint", linetype = "Scenario") +
  scale_color_manual(values = rp_cols) +
  ggplot2::theme_classic(base_size = 8) +
  theme(legend.position = "bottom",
        strip.background = element_blank(),
        panel.background = element_blank(),
        plot.background = element_blank(),
        legend.background = element_blank(),
        legend.box.background = element_blank(),
        plot.margin = margin(c(5.5, 10, 5.5, 5.5))) %>%
  suppressWarnings()

ggsave(
  plot = print(p),
  filename = file.path(base_dir, "figures/Short-term_projections.png"),
  units = 'in', width = pagewidth_in, height = pagewidth_in*0.6,
  bg = "white"
) %>%
  suppressWarnings()

### NFD projections 2024: >=45 cm biomass ####

line_types <- c("MSY" = 1, "0" = 2, "No Btrigger" = 3, 4)
names(line_types)[4] <- paste("Assumed", start_year, "catch")

p <-
  bind_rows(
    plot_biomass(proj_hr0_fit, min_catch_length = 45, return_data = TRUE) %>%
      dplyr::select(year, value) %>%
      # filter(year <= start_year+3) %>%
      mutate(HR = "0"),
    plot_biomass(proj_msy_fit, min_catch_length = 45, return_data = TRUE) %>%
      dplyr::select(year, value) %>%
      # filter(year <= start_year+3) %>%
      mutate(HR = "MSY"),
    plot_biomass(proj_msy_nobtrigger_fit, min_catch_length = 45, return_data = TRUE) %>%
      dplyr::select(year, value) %>%
      # filter(year <= start_year+3) %>%
      mutate(HR = "No Btrigger"),
    plot_biomass(proj_hrly_fit, min_catch_length = 45, return_data = TRUE) %>%
      dplyr::select(year, value) %>%
      # filter(year <= start_year+3) %>%
      mutate(HR = paste("Assumed", start_year, "catch"))
  ) %>%
  ggplot(ggplot2::aes(.data$year, .data$value, linetype = .data$HR)) +
  ggplot2::geom_line() +
  ggplot2::expand_limits(y = 0) +
  coord_cartesian(expand = FALSE, xlim = c(min_year, NA)) +
  ggplot2::scale_x_continuous(breaks = seq(1980,2030,1)) +
  scale_linetype_manual(values = line_types) +
  labs(x = "Year", y = expression("Biomass of ">="45 cm fish (kt)"),
       linetype = "Scenario") +
  ggplot2::theme_classic(base_size = 8) +
  theme(legend.position = "bottom",
        strip.background = element_blank(),
        panel.background = element_blank(),
        plot.background = element_blank(),
        legend.background = element_blank(),
        legend.box.background = element_blank(),
        plot.margin = margin(c(5.5, 10, 5.5, 5.5))) %>%
  suppressWarnings()

ggsave(
  plot = print(p),
  filename = file.path(base_dir, "figures/Short-term_projection_45cm.png"),
  units = 'in', width = pagewidth_in, height = pagewidth_in*0.6,
  bg = "white"
) %>%
  suppressWarnings()

### NFD projections 2024: catch ####

line_types <- c("MSY" = 1, "0" = 2, "No Btrigger" = 3, 4)
names(line_types)[4] <- paste("Assumed", start_year, "catch")

p <-
  bind_rows(
    plot_catch(proj_hr0_fit, return_data = TRUE) %>%
      group_by(year) %>%
      reframe(value = sum(value, na.rm = TRUE)) %>%
      # filter(year <= start_year+3) %>%
      mutate(HR = "0"),
    plot_catch(proj_msy_fit, return_data = TRUE) %>%
      group_by(year) %>%
      reframe(value = sum(value, na.rm = TRUE)) %>%
      # filter(year <= start_year+3) %>%
      mutate(HR = "MSY"),
    plot_catch(proj_msy_nobtrigger_fit, return_data = TRUE) %>%
      group_by(year) %>%
      reframe(value = sum(value, na.rm = TRUE)) %>%
      # filter(year <= start_year+3) %>%
      mutate(HR = "No Btrigger"),
    plot_catch(proj_hrly_fit, return_data = TRUE) %>%
      group_by(year) %>%
      reframe(value = sum(value, na.rm = TRUE)) %>%
      # filter(year <= start_year+3) %>%
      mutate(HR = paste("Assumed", start_year, "catch"))
  ) %>%
  ggplot(ggplot2::aes(.data$year, .data$value, linetype = .data$HR)) +
  ggplot2::geom_line() +
  ggplot2::expand_limits(y = 0) +
  coord_cartesian(expand = FALSE, xlim = c(min_year, NA)) +
  ggplot2::scale_x_continuous(breaks = seq(1980,2030,1)) +
  scale_linetype_manual(values = line_types) +
  labs(x = "Year", y = "Catch (kt)", linetype = "Scenario") +
  ggplot2::theme_classic(base_size = 8) +
  theme(legend.position = "bottom",
        strip.background = element_blank(),
        panel.background = element_blank(),
        plot.background = element_blank(),
        legend.background = element_blank(),
        legend.box.background = element_blank(),
        plot.margin = margin(c(5.5, 10, 5.5, 5.5))) %>%
  suppressWarnings()

ggsave(
  plot = print(p),
  filename = file.path(base_dir, "figures/Short-term_projection_catch.png"),
  units = 'in', width = pagewidth_in, height = pagewidth_in*0.6,
  bg = "white"
) %>%
  suppressWarnings()

# NFD projection plot

# line_types <- c("MSY" = 1, "25 kt" = 2, "30 kt" = 3)
#
# p <-
#   bind_rows(
#     plot_biomass(proj_msy_fit, stocks = "ghl_female_mat", return_data = TRUE) %>%
#       dplyr::select(year, value) %>%
#       mutate(HR = "MSY"),
#     plot_biomass(proj_25kt_fit, stocks = "ghl_female_mat", return_data = TRUE) %>%
#       dplyr::select(year, value) %>%
#       mutate(HR = "25 kt"),
#     plot_biomass(proj_30kt_fit, stocks = "ghl_female_mat", return_data = TRUE) %>%
#       dplyr::select(year, value) %>%
#       mutate(HR = "30 kt")
#   ) %>%
#   ggplot(ggplot2::aes(.data$year, .data$value, linetype = .data$HR)) +
#   ggplot2::geom_line() +
#   geom_hline(data = rp_tab %>%
#                filter(rp %in% c("Bpa", "Blim")),
#              aes(yintercept = Value, color = rp)) +
#   ggplot2::expand_limits(y = 0) +
#   ggplot2::coord_cartesian(expand = FALSE) +
#   ggplot2::scale_x_continuous(breaks = seq(1980,2030,2)) +
#   scale_linetype_manual(values = line_types) +
#   labs(x = "Year", y = "Female spawning stock biomass (kt)", color =
#          "Reference\npoint",
#        linetype = "Harvest\nscenario") +
#   scale_color_manual(values = rp_cols) +
#   ggplot2::theme_classic(base_size = 8) +
#   theme(legend.position = "bottom") %>%
#   suppressWarnings()
#
# ggsave(
#   plot = print(p),
#   filename = file.path(base_dir, "figures/NFD_projections.png"),
#   units = 'in', width = pagewidth_in, height = pagewidth_in*0.6,
#   bg = "white"
# ) %>%
#   suppressWarnings()



# Tables ####

### Basis table ####

basis_tab <-
  tibble(
    Variable =
      c(
        paste0("Harvest rate >= 45 cm (", start_year, ")"),
        paste0("Biomass >= 45 cm (", start_year + 1, ")"),
        paste0("SSB (", start_year + 1, ")"),
        paste0("Recruitment (", start_year, "-", start_year + 2, ")"),
        paste0("Expected catch (", start_year, ")")
      ),
    Value = c(
      plot_hr(proj_msy_fit, min_catch_length = 45, return_data = T) %>%
        filter(year == start_year) %>%
        pull(value) %>% round(., 3),
      plot_biomass(proj_msy_fit, min_catch_length = 45, return_data = TRUE) %>%
        filter(year == start_year + 1) %>%
        pull(total.biomass) %>% {./1e6},
      plot_biomass(proj_msy_fit, stocks = "ghl_female_mat",
                   return_data = TRUE) %>%
        filter(year == start_year + 1) %>%
        pull(total.biomass) %>% {./1e6},
      mean(bootstr_rec$recruitment)/1e6,
      sum(expected_catch$catch)/1e6
    ),
    Notes = c(
      paste0("Based on expected catch in ", start_year),
      paste0("Beginning of ", start_year + 1, "; kilotonnes"),
      paste0("Beginning of ", start_year + 1, "; kilotonnes"),
      paste0("Average ", paste(range(bootstr_rec$year), collapse = "-"), " recruitment in millions. Does not influence the short-term forecasts"),
      paste0("TAC for ", start_year, " plus mean catch above TAC 2018-2023 (1800 t); kilotonnes")
    )
  )

## Values for advice tables ####

ref_ssb <- basis_tab %>% filter(grepl("SSB", Variable)) %>% pull(Value)*1e3

tmp <-
  bind_rows(

    ## MSY scenario
    full_join(
      plot_hr(proj_msy_fit, min_catch_length = 45, return_data = TRUE) %>%
        filter(year %in% (start_year-1):(start_year+3)) %>%
        dplyr::select(-step, -area) %>%
        mutate(type = "msy", .before = 1),
      plot_biomass(proj_msy_fit, stocks = "ghl_female_mat", return_data = TRUE) %>%
        filter(year %in% (start_year-1):(start_year+3)) %>%
        mutate(ssb = total.biomass/1e3) %>%
        dplyr::select(year, ssb),
      by = "year") %>%
      mutate(
        below_bpa = ssb < bpa/1e3,
        d_ssb = 100*(ssb-ref_ssb)/ref_ssb,
        d_advice = 100*(catch_biom/(last_advice*1e3)-1),
        d_tac = 100*(catch_biom/(last_tac*1e3)-1),
        d_biom = 100*(c(0, diff(biomass))/biomass)),

    ## MSY without Btrigger
    full_join(
      plot_hr(proj_msy_nobtrigger_fit, min_catch_length = 45, return_data = TRUE) %>%
        filter(year %in% (start_year-1):(start_year+3)) %>%
        dplyr::select(-step, -area) %>%
        mutate(type = "msy_nobtrigger", .before = 1),
      plot_biomass(proj_msy_nobtrigger_fit, stocks = "ghl_female_mat",
                   return_data = TRUE) %>%
        filter(year %in% (start_year-1):(start_year+3)) %>%
        mutate(ssb = total.biomass/1e3) %>%
        dplyr::select(year, ssb),
      by = "year") %>%
      mutate(
        below_bpa = ssb < bpa/1e3,
        d_ssb = 100*(ssb-ref_ssb)/ref_ssb,
        d_advice = 100*(catch_biom/(last_advice*1e3)-1),
        d_tac = 100*(catch_biom/(last_tac*1e3)-1),
        d_biom = 100*(c(0, diff(biomass))/biomass)),

    ## MSY with intermediate catch of 23 kt
    # full_join(
    #   plot_hr(proj_tac_fit, min_catch_length = 45, return_data = TRUE) %>%
    #     filter(year %in% (start_year-1):(start_year+3)) %>%
    #     dplyr::select(-step, -area) %>%
    #     mutate(type = "msy_int", .before = 1),
    #   plot_biomass(proj_tac_fit, stocks = "ghl_female_mat", return_data = TRUE) %>%
    #     filter(year %in% (start_year-1):(start_year+3)) %>%
    #     mutate(ssb = total.biomass/1e3) %>%
    #     dplyr::select(year, ssb),
    #   by = "year") %>%
    #   mutate(
    #     below_bpa = ssb < bpa/1e3,
    #     d_ssb = 100*(c(0, diff(ssb))/ssb),
    #     d_advice = 100*(catch_biom/(last_advice*1e3)-1),
    #     d_tac = 100*(catch_biom/(last_tac*1e3)-1),
    #     d_biom = 100*(c(0, diff(biomass))/biomass)),

    ## No fishing scenario
    full_join(
      plot_hr(proj_hr0_fit, min_catch_length = 45, return_data = TRUE) %>%
        filter(year %in% (start_year-1):(start_year+3)) %>%
        dplyr::select(-step, -area) %>%
        mutate(type = "0", .before = 1),
      plot_biomass(proj_hr0_fit, stocks = "ghl_female_mat", return_data = TRUE) %>%
        filter(year %in% (start_year-1):(start_year+3)) %>%
        mutate(ssb = total.biomass/1e3) %>%
        dplyr::select(year, ssb),
      by = "year") %>%
      mutate(
        below_bpa = ssb < bpa/1e3,
        d_ssb = 100*(ssb-ref_ssb)/ref_ssb,
        d_advice = 100*(catch_biom/(last_advice*1e3)-1),
        d_tac = 100*(catch_biom/(last_tac*1e3)-1),
        d_biom = 100*(c(0, diff(biomass))/biomass)),

    ## Catch 2024 scenario
    full_join(
      plot_hr(proj_hrly_fit, min_catch_length = 45, return_data = TRUE) %>%
        filter(year %in% (start_year-1):(start_year+3)) %>%
        dplyr::select(-step, -area) %>%
        mutate(type = "ly", .before = 1),
      plot_biomass(proj_hrly_fit, stocks = "ghl_female_mat", return_data = TRUE) %>%
        filter(year %in% (start_year-1):(start_year+3)) %>%
        mutate(ssb = total.biomass/1e3) %>%
        dplyr::select(year, ssb),
      by = "year") %>%
      mutate(
        below_bpa = ssb < bpa/1e3,
        d_ssb = 100*(ssb-ref_ssb)/ref_ssb,
        d_advice = 100*(catch_biom/(last_advice*1e3)-1),
        d_tac = 100*(catch_biom/(last_tac*1e3)-1),
        d_biom = 100*(c(0, diff(biomass))/biomass)),
  )


## Advice for the next year table ####

next_year_tab <-
  tibble(
    Basis =
      c(paste0("Advice basis for ", start_year + 1),
        paste0("HRmsy"),
        paste0("Other scenarios for ", start_year + 1),
        # "HRmsy 23 kt 2024 catch",
        "HRmsy no advice rule",
        "HR = 0",
        "Status quo catch"
      ),
    Catch =
      c("",
        tmp %>%
          filter(type == "msy", year == start_year + 1) %>%
          pull(catch_biom) %>% {./1e3} %>% round(),
        "",
        # tmp %>%
        #   filter(type == "msy_int", year == start_year + 1) %>%
        #   pull(catch_biom) %>% {./1e3} %>% round(),
        tmp %>%
          filter(type == "msy_nobtrigger", year == start_year + 1) %>%
          pull(catch_biom) %>% {./1e3} %>% round(),
        tmp %>%
          filter(type == "0", year == start_year + 1) %>%
          pull(catch_biom) %>% {./1e3} %>% round(),
        tmp %>%
          filter(type == "ly", year == start_year + 1) %>%
          pull(catch_biom) %>% {./1e3} %>% round()
      ),
    HR =
      c("",
        tmp %>%
          filter(type == "msy", year == start_year + 1) %>%
          pull(value) %>% round(3),
        "",
        # tmp %>%
        #   filter(type == "msy_int", year == start_year + 1) %>%
        #   pull(value) %>% round(3),
        tmp %>%
          filter(type == "msy_nobtrigger", year == start_year + 1) %>%
          pull(value) %>% round(3),
        tmp %>%
          filter(type == "0", year == start_year + 1) %>%
          pull(value) %>% round(3),
        tmp %>%
          filter(type == "ly", year == start_year + 1) %>%
          pull(value) %>% round(3)
      ),
    `SSB` =
      c("",
        tmp %>%
          filter(type == "msy", year == start_year + 2) %>%
          pull(ssb) %>% round(),
        "",
        # tmp %>%
        #   filter(type == "msy_int", year == start_year + 2) %>%
        #   pull(ssb) %>% round(),
        tmp %>%
          filter(type == "msy_nobtrigger", year == start_year + 2) %>%
          pull(ssb) %>% round(),
        tmp %>%
          filter(type == "0", year == start_year + 2) %>%
          pull(ssb) %>% round(),
        tmp %>%
          filter(type == "ly", year == start_year + 2) %>%
          pull(ssb) %>% round()
      ),
    `SSB_below_Bpa`=
      c("",
        tmp %>%
          filter(type == "msy", year == start_year + 2) %>%
          pull(below_bpa) %>% round(),
        "",
        # tmp %>%
        #   filter(type == "msy_int", year == start_year + 2) %>%
        #   pull(below_bpa) %>% round(),
        tmp %>%
          filter(type == "msy_nobtrigger", year == start_year + 2) %>%
          pull(below_bpa) %>% round(),
        tmp %>%
          filter(type == "0", year == start_year + 2) %>%
          pull(below_bpa) %>% round(),
        tmp %>%
          filter(type == "ly", year == start_year + 2) %>%
          pull(below_bpa) %>% round()
      ),
    `SSB change` =
      c("",
        tmp %>%
          filter(type == "msy", year == start_year + 2) %>%
          pull(d_ssb) %>% round(1),
        "",
        # tmp %>%
        #   filter(type == "msy_int", year == start_year + 2) %>%
        #   pull(d_ssb) %>% round(1),
        tmp %>%
          filter(type == "msy_nobtrigger", year == start_year + 2) %>%
          pull(d_ssb) %>% round(1),
        tmp %>%
          filter(type == "0", year == start_year + 2) %>%
          pull(d_ssb) %>% round(1),
        tmp %>%
          filter(type == "ly", year == start_year + 2) %>%
          pull(d_ssb) %>% round(1)
      ),
    `TAC change` =
      c("",
        tmp %>%
          filter(type == "msy", year == start_year + 1) %>%
          pull(d_tac) %>% round(1),
        "",
        # tmp %>%
        #   filter(type == "msy_int", year == start_year + 1) %>%
        #   pull(d_tac) %>% round(1),
        tmp %>%
          filter(type == "msy_nobtrigger", year == start_year + 1) %>%
          pull(d_tac) %>% round(1),
        tmp %>%
          filter(type == "0", year == start_year + 1) %>%
          pull(d_tac) %>% round(1),
        tmp %>%
          filter(type == "ly", year == start_year + 1) %>%
          pull(d_tac) %>% round(1)
      ),
    `Advice change` =
      c("",
        tmp %>%
          filter(type == "msy", year == start_year + 1) %>%
          pull(d_advice) %>% round(1),
        "",
        # tmp %>%
        #   filter(type == "msy_int", year == start_year + 1) %>%
        #   pull(d_advice) %>% round(1),
        tmp %>%
          filter(type == "msy_nobtrigger", year == start_year + 1) %>%
          pull(d_advice) %>% round(1),
        tmp %>%
          filter(type == "0", year == start_year + 1) %>%
          pull(d_advice) %>% round(1),
        tmp %>%
          filter(type == "ly", year == start_year + 1) %>%
          pull(d_advice) %>% round(1)
      )
  )

## Advice for the year after table ####

last_year_tab <-
  tibble(
    Basis =
      c(paste0("Advice basis for ", start_year + 2),
        paste0("HRmsy"),
        paste0("Other scenarios for ", start_year + 2),
        # "HRmsy 23 kt 2024 catch",
        "HRmsy no advice rule",
        "HR = 0",
        paste0("Status quo catch")
      ),
    Catch =
      c("",
        tmp %>%
          filter(type == "msy", year == start_year + 2) %>%
          pull(catch_biom) %>% {./1e3} %>% round(),
        "",
        # tmp %>%
        #   filter(type == "msy_int", year == start_year + 2) %>%
        #   pull(catch_biom) %>% {./1e3} %>% round(),
        tmp %>%
          filter(type == "msy_nobtrigger", year == start_year + 2) %>%
          pull(catch_biom) %>% {./1e3} %>% round(),
        tmp %>%
          filter(type == "0", year == start_year + 2) %>%
          pull(catch_biom) %>% {./1e3} %>% round(),
        tmp %>%
          filter(type == "ly", year == start_year + 2) %>%
          pull(catch_biom) %>% {./1e3} %>% round()
      ),
    HR =
      c("",
        tmp %>%
          filter(type == "msy", year == start_year + 2) %>%
          pull(value) %>% round(3),
        "",
        # tmp %>%
        #   filter(type == "msy_int", year == start_year + 2) %>%
        #   pull(value) %>% round(3),
        tmp %>%
          filter(type == "msy_nobtrigger", year == start_year + 2) %>%
          pull(value) %>% round(3),
        tmp %>%
          filter(type == "0", year == start_year + 2) %>%
          pull(value) %>% round(3),
        tmp %>%
          filter(type == "ly", year == start_year + 2) %>%
          pull(value) %>% round(3)
      ),
    `SSB` =
      c("",
        tmp %>%
          filter(type == "msy", year == start_year + 3) %>%
          pull(ssb) %>% round(),
        "",
        # tmp %>%
        #   filter(type == "msy_int", year == start_year + 3) %>%
        #   pull(ssb) %>% round(),
        tmp %>%
          filter(type == "msy_nobtrigger", year == start_year + 3) %>%
          pull(ssb) %>% round(),
        tmp %>%
          filter(type == "0", year == start_year + 3) %>%
          pull(ssb) %>% round(),
        tmp %>%
          filter(type == "ly", year == start_year + 3) %>%
          pull(ssb) %>% round()
      ),
    `SSB_below_Bpa`=
      c("",
        tmp %>%
          filter(type == "msy", year == start_year + 3) %>%
          pull(below_bpa) %>% round(),
        "",
        # tmp %>%
        #   filter(type == "msy_int", year == start_year + 3) %>%
        #   pull(below_bpa) %>% round(),
        tmp %>%
          filter(type == "msy_nobtrigger", year == start_year + 3) %>%
          pull(below_bpa) %>% round(),
        tmp %>%
          filter(type == "0", year == start_year + 3) %>%
          pull(below_bpa) %>% round(),
        tmp %>%
          filter(type == "ly", year == start_year + 3) %>%
          pull(below_bpa) %>% round()
      ),
    `SSB change` =
      c("",
        tmp %>%
          filter(type == "msy", year == start_year + 3) %>%
          pull(d_ssb) %>% round(1),
        "",
        # tmp %>%
        #   filter(type == "msy_int", year == start_year + 3) %>%
        #   pull(d_ssb) %>% round(1),
        tmp %>%
          filter(type == "msy_nobtrigger", year == start_year + 3) %>%
          pull(d_ssb) %>% round(1),
        tmp %>%
          filter(type == "0", year == start_year + 3) %>%
          pull(d_ssb) %>% round(1),
        tmp %>%
          filter(type == "ly", year == start_year + 3) %>%
          pull(d_ssb) %>% round(1)
      ),
    `TAC change` =
      c("",
        tmp %>%
          filter(type == "msy", year == start_year + 2) %>%
          pull(d_tac) %>% round(1),
        "",
        # tmp %>%
        #   filter(type == "msy_int", year == start_year + 2) %>%
        #   pull(d_tac) %>% round(1),
        tmp %>%
          filter(type == "msy_nobtrigger", year == start_year + 2) %>%
          pull(d_tac) %>% round(1),
        tmp %>%
          filter(type == "0", year == start_year + 2) %>%
          pull(d_tac) %>% round(1),
        tmp %>%
          filter(type == "ly", year == start_year + 2) %>%
          pull(d_tac) %>% round(1)
      ),
    `Advice change` =
      c("",
        tmp %>%
          filter(type == "msy", year == start_year + 2) %>%
          pull(d_advice) %>% round(1),
        "",
        # tmp %>%
        #   filter(type == "msy_int", year == start_year + 2) %>%
        #   pull(d_advice) %>% round(1),
        tmp %>%
          filter(type == "msy_nobtrigger", year == start_year + 2) %>%
          pull(d_advice) %>% round(1),
        tmp %>%
          filter(type == "0", year == start_year + 2) %>%
          pull(d_advice) %>% round(1),
        tmp %>%
          filter(type == "ly", year == start_year + 2) %>%
          pull(d_advice) %>% round(1)
      )
  )


## Save ####

advice_calculations <- tmp

save(advice_calculations, basis_tab, last_year_tab, next_year_tab, last_advice, last_tac, start_year, end_year, file = file.path(outpath, 'short_term_projection_tables.Rdata'), compress = "xz")

save(proj_msy_fit, proj_msy_nobtrigger_fit,  proj_hr0_fit, proj_hrly_fit, base.par.proj, file = file.path(outpath, 'short_term_projection_fit_objects.Rdata'), compress = "xz")

