## ---------------------------
##
## Script name: Compile/read survey index data from the data folder
##
## ---------------------------

## Source or list custom functions used within the script

## ---------------------------

# EggaN ####

message("EggaN indices")

EggaN_SI_as_biomass_index <- TRUE # Switch to shift between abundance and biomass indices

### Females ####

EggaN_SI_female <- lapply(EggaN, function(k) k$station_length) %>%
  bind_rows() %>%
  filter(
    Stratum %in% c("C500-700", "C700-1000", "D500-700", "D700-1000",
                   "E500-700", "E700-1000", "F500-700", "F700-1000"),
    IndividualTotalLength >= 28,
    IndividualSex == "F",
    year %in% model_params$year_range[model_params$year_range >= 1996]
  ) %>%
  group_by(year) %>%
  {if(EggaN_SI_as_biomass_index) summarise(., weight = sum(Biomass, na.rm = TRUE)) else summarise(., number = sum(Abundance, na.rm = TRUE))} %>%
  dplyr::mutate(step = "1", length = "len28", .after = "year") %>%
  add_g3_attributes(
    list(length = mfdb_interval(
      "len", c(28, stock_params$maxlength),
      open_ended = c("upper")),
      timestep = model_params$timestep_fun),
    warn = 2
  )

### Males ####

EggaN_SI_male <- lapply(EggaN, function(k) k$station_length) %>%
  bind_rows() %>%
  filter(
    Stratum %in% c("C500-700", "C700-1000", "D500-700", "D700-1000",
                   "E500-700", "E700-1000", "F500-700", "F700-1000"),
    IndividualTotalLength >= 28,
    IndividualSex == "M",
    year %in% model_params$year_range[model_params$year_range >= 1996]
  ) %>%
  group_by(year) %>%
  {if(EggaN_SI_as_biomass_index) summarise(., weight = sum(Biomass, na.rm = TRUE)) else summarise(., number = sum(Abundance, na.rm = TRUE))} %>%
  dplyr::mutate(step = "1", length = "len28", .after = "year") %>%
  add_g3_attributes(
    list(length = mfdb_interval(
      "len", c(28, stock_params$maxlength),
      open_ended = c("upper")),
      timestep = model_params$timestep_fun),
    warn = 2
  )

### Combined ####

EggaN_SI <- lapply(EggaN, function(k) k$station_length) %>%
  bind_rows() %>%
  filter(
    Stratum %in% c("C500-700", "C700-1000", "D500-700", "D700-1000",
                   "E500-700", "E700-1000", "F500-700", "F700-1000"),
    IndividualTotalLength >= 28,
    year %in% model_params$year_range[model_params$year_range >= 1996]
  )

po <- EggaN_SI %>%
  {if(EggaN_SI_as_biomass_index) rename(., value = Biomass) else rename(., value = Abundance)} %>%
  group_by(year, HaulKey) %>%
  summarise(value = sum(value)) %>%
  ggplot(aes(x = year, y = value/1e6)) +
  geom_smooth(method = 'loess', formula = 'y ~ x') +
  geom_boxplot(aes(group = year), alpha = 0.5) +
  labs(
    y = ifelse(EggaN_SI_as_biomass_index,
               "Biomass contribution per station (kt)",
               "Abundance contribution per station (millions)"),
    x = "Year") +
  scale_x_continuous(expand = c(0, 0), breaks = seq(1900, 2030, 2)) +
  scale_y_continuous(expand = expansion(mult = c(0, 0.01))) +
  expand_limits(y = 0)

pol <- EggaN_SI %>%
  {if(EggaN_SI_as_biomass_index) rename(., value = Biomass) else rename(., value = Abundance)} %>%
  group_by(year, HaulKey) %>%
  summarise(value = sum(value)) %>%
  ggplot(aes(x = year, y = value/1e6)) +
  geom_smooth(method = 'loess', formula = 'y ~ x') +
  geom_boxplot(aes(group = year), alpha = 0.5) +
  labs(
    y = NULL,
    x = "Year") +
  scale_x_continuous(expand = c(0, 0), breaks = seq(1900, 2030, 2)) +
  scale_y_log10(expand = expansion(mult = c(0, 0.01)))

EggaN_SI <- EggaN_SI %>%
  group_by(year) %>%
  {if(EggaN_SI_as_biomass_index) summarise(., weight = sum(Biomass, na.rm = TRUE)) else summarise(., number = sum(Abundance, na.rm = TRUE))} %>%
  dplyr::mutate(step = "1", length = "len28", .after = "year") %>%
  add_g3_attributes(
    list(length = mfdb_interval(
      "len", c(28, stock_params$maxlength),
      open_ended = c("upper")),
      timestep = model_params$timestep_fun),
    warn = 2
  )

png(file.path(base_dir, "figures/EggaN_SI_outliers.png"), width = pagewidth, height = pagewidth*1.2, units = "mm", res = 300)
print(cowplot::plot_grid(
  cowplot::plot_grid(po + ggtitle("Per station"), pol + ggtitle("Per station, log scale"), ncol = 2),
  ggplot(
    EggaN_SI %>%
      mutate(value = if(EggaN_SI_as_biomass_index) weight else number) ,
    aes(x = year, y = value/1e6)
  ) +
    geom_col() +
    labs(
      y = if(EggaN_SI_as_biomass_index) "Survey index biomass (kt)" else {
        "Survey index abundance (millions)"
      },
      x = "Year") +
    scale_x_continuous(expand = c(0, 0), breaks = seq(1900, 2030, 2)) +
    scale_y_continuous(expand = c(0, 0)) +
    ggtitle("Index"),
  ncol = 1
))
dev.off()

#### Plot ####

p <- bind_rows(EggaN_SI_female %>% mutate(sex = "F"),
               EggaN_SI_male %>% mutate(sex = "M")
) %>%
  mutate(value = if(EggaN_SI_as_biomass_index) weight else number) %>%
  ggplot(., aes(x = year, y = value/1e6, fill = sex)) +
  geom_col(position = position_stack(reverse = TRUE)) +
  scale_x_continuous("Year", expand = c(0, 0), breaks = seq(1900, 2030, 2)) +
  scale_y_continuous(
    if(EggaN_SI_as_biomass_index) "Survey index biomass (kt)" else {
      "Survey index abundance (millions)"
    },
    expand = c(0, 0)) +
  scale_fill_manual("Fill", values = c("F" = "#FF5F68", "M" = "#449BCF")) +
  theme(legend.position = "bottom")

png(file.path(base_dir, "figures/EggaN_index.png"), width = pagewidth, height = pagewidth*0.7, units = "mm", res = 300)
print(p)
dev.off()

#### Plot with confidence intervals ####

if(EggaN_SI_as_biomass_index) {
  if(all(sapply(EggaN, function(k) "bs_biomass_year" %in% names(k)))) {
    bs_EggaN_SI <- lapply(EggaN, function(k) k$bs_biomass_year) %>%
      bind_rows() %>%
      filter(!is.na(IndividualSex))
  } else {
    bs_EggaN_SI <- NULL
  }
} else {
  if(all(sapply(BESS, function(k) "bs_abundance_year" %in% names(k)))) {
    bs_EggaN_SI <- lapply(EggaN, function(k) k$bs_abundance_year) %>%
      bind_rows() %>%
      filter(!is.na(IndividualSex))
  } else {
    bs_EggaN_SI <- NULL
  }
}

if(!is.null(bs_EggaN_SI)) {
  p <- bs_EggaN_SI %>%
    {if(EggaN_SI_as_biomass_index) {
      rename(., value = `Biomass_sum_50%`,
             upr = `Biomass_sum_97.5%`,
             lwr = `Biomass_sum_2.5%`)
    } else {
      rename(., value = `Abundance_sum_50%`,
             upr = `Abundance_sum_97.5%`,
             lwr = `Abundance_sum_2.5%`)
    }} %>%
    mutate(year = as.integer(year)) %>%
    group_by(year, IndividualSex) %>%
    summarise(value = sum(value, na.rm = TRUE)/1e6,
              upr = sum(upr, na.rm = TRUE)/1e6,
              lwr = sum(lwr, na.rm = TRUE)/1e6) %>%
    ggplot(aes(x = year, y = value, ymin = upr, ymax = lwr,
               color = IndividualSex, fill = IndividualSex)) +
    geom_ribbon(alpha = 0.3, color = NA) +
    geom_path() +
    geom_point(shape = 1) +
    geom_path(
      data = bind_rows(EggaN_SI_female %>% mutate(sex = "F"),
                       EggaN_SI_male %>% mutate(sex = "M")
      ) %>%
        mutate(value = if(EggaN_SI_as_biomass_index) weight else number),
      aes(x = year, y = value/1e6, color = sex),
      linetype = 2, inherit.aes = FALSE
    ) +
    labs(
      y = ifelse(EggaN_SI_as_biomass_index,
                 "Survey index biomass ± 95% CIs (kt)",
                 "Survey index abundance ± 95% CIs (millions)"),
      x = "Year",
      fill = "Sex", color = "Sex") +
    scale_fill_manual(values = c("F" = "#FF5F68", "M" = "#449BCF")) +
    scale_color_manual(values = c("F" = "#FF5F68", "M" = "#449BCF")) +
    expand_limits(y = 0) +
    scale_x_continuous(expand = c(0, 0), breaks = seq(1900, 2030, 2)) +
    scale_y_continuous(expand = c(0, 0)) +
    coord_cartesian(clip = "off") +
    theme(legend.position = "bottom")

  png(file.path(base_dir, "figures/EggaN_index_with_uncertainty.png"), width = pagewidth, height = pagewidth*0.7, units = "mm", res = 300)
  print(p)
  dev.off()
}

# EggaS ####

message("EggaS indices")

EggaS_SI_as_biomass_index <- FALSE # Switch to shift between abundance and biomass indices

### Females ####

EggaS_SI_female <- lapply(EggaS, function(k) k$station_length) %>%
  bind_rows() %>%
  filter(
    IndividualTotalLength >= 28,
    IndividualSex == "F",
    year %in% model_params$year_range[model_params$year_range != 2020]
  ) %>%
  group_by(year) %>%
  {if(EggaS_SI_as_biomass_index) summarise(., weight = sum(Biomass, na.rm = TRUE)) else summarise(., number = sum(Abundance, na.rm = TRUE))} %>%
  dplyr::mutate(step = "1", length = "len28", .after = "year") %>%
  add_g3_attributes(
    list(length = mfdb_interval(
      "len", c(28, stock_params$maxlength),
      open_ended = c("upper")),
      timestep = model_params$timestep_fun),
    warn = 2
  )

### Males ####

EggaS_SI_male <- lapply(EggaS, function(k) k$station_length) %>%
  bind_rows() %>%
  filter(
    IndividualTotalLength >= 28,
    IndividualSex == "M",
    year %in% model_params$year_range[model_params$year_range != 2020]
  ) %>%
  group_by(year) %>%
  {if(EggaS_SI_as_biomass_index) summarise(., weight = sum(Biomass, na.rm = TRUE)) else summarise(., number = sum(Abundance, na.rm = TRUE))} %>%
  dplyr::mutate(step = "1", length = "len28", .after = "year") %>%
  add_g3_attributes(
    list(length = mfdb_interval(
      "len", c(28, stock_params$maxlength),
      open_ended = c("upper")),
      timestep = model_params$timestep_fun),
    warn = 2
  )

### Combined ####

EggaS_SI <- lapply(EggaS, function(k) k$station_length) %>%
  bind_rows() %>%
  filter(
    IndividualTotalLength >= 28,
    year %in% model_params$year_range[model_params$year_range != 2020]
  )

po <- EggaS_SI %>%
  rename(., value = Abundance) %>%
  group_by(year, HaulKey) %>%
  summarise(value = sum(value)) %>%
  ggplot(aes(x = year, y = value/1e6)) +
  geom_smooth(method = 'loess', formula = 'y ~ x') +
  geom_boxplot(aes(group = year), alpha = 0.5) +
  labs(
    y = "Abundance contribution per station (millions)",
    x = "Year") +
  scale_x_continuous(expand = c(0, 0), breaks = seq(1900, 2030, 2)) +
  scale_y_continuous(expand = expansion(mult = c(0, 0.01))) +
  expand_limits(y = 0)

pol <- EggaS_SI %>%
  rename(., value = Abundance) %>%
  group_by(year, HaulKey) %>%
  summarise(value = sum(value)) %>%
  ggplot(aes(x = year, y = value/1e6)) +
  geom_smooth(method = 'loess', formula = 'y ~ x') +
  geom_boxplot(aes(group = year), alpha = 0.5) +
  labs(
    y = NULL,
    x = "Year") +
  scale_x_continuous(expand = c(0, 0), breaks = seq(1900, 2030, 2)) +
  scale_y_log10(expand = expansion(mult = c(0, 0.01)))

EggaS_SI <- EggaS_SI %>%
  group_by(year) %>%
  {if(EggaS_SI_as_biomass_index) summarise(., weight = sum(Biomass, na.rm = TRUE)) else summarise(., number = sum(Abundance, na.rm = TRUE))} %>%
  dplyr::mutate(step = "1", length = "len28", .after = "year") %>%
  add_g3_attributes(
    list(length = mfdb_interval(
      "len", c(28, stock_params$maxlength),
      open_ended = c("upper")),
      timestep = model_params$timestep_fun),
    warn = 2
  )

png(file.path(base_dir, "figures/EggaS_SI_outliers.png"), width = pagewidth, height = pagewidth*1.2, units = "mm", res = 300)
print(cowplot::plot_grid(
  cowplot::plot_grid(po + ggtitle("Per station"), pol + ggtitle("Per station, log scale"), ncol = 2),
  ggplot(
    EggaS_SI %>%
      mutate(value = if(EggaS_SI_as_biomass_index) weight else number) ,
    aes(x = year, y = value/1e6)
  ) +
    geom_col() +
    labs(
      y = if(EggaS_SI_as_biomass_index) "Survey index biomass (kt)" else {
        "Survey index abundance (millions)"
      },
      x = "Year") +
    scale_x_continuous(expand = c(0, 0), breaks = seq(1900, 2030, 2)) +
    scale_y_continuous(expand = c(0, 0)) +
    ggtitle("Index"),
  ncol = 1
))
dev.off()

#### Plot ####

# p <- bind_rows(EggaS_SI_female %>% mutate(sex = "F"),
#                EggaS_SI_male %>% mutate(sex = "M")
# ) %>%
#   mutate(value = if(EggaS_SI_as_biomass_index) weight else number) %>%
#   ggplot(., aes(x = year, y = value/1e6, fill = sex)) +
#   geom_area(position = position_stack(reverse = TRUE)) +
#   scale_x_continuous("Year", expand = c(0, 0), breaks = seq(1900, 2030, 2)) +
#   scale_y_continuous(
#     if(EggaS_SI_as_biomass_index) "Survey index biomass (kt)" else {
#       "Survey index abundance (millions)"
#     },
#     expand = c(0, 0)) +
#   labs(fill = "Sex") +
#   theme(legend.position = "bottom")

p <- ggplot(
  EggaS_SI %>%
    mutate(value = if(EggaS_SI_as_biomass_index) weight else number) ,
  aes(x = year, y = value/1e6)
) +
  geom_col() +
  labs(
    y = if(EggaS_SI_as_biomass_index) "Survey index biomass (kt)" else {
      "Survey index abundance (millions)"
    },
    x = "Year") +
  scale_x_continuous(expand = c(0, 0), breaks = seq(1900, 2030, 2)) +
  scale_y_continuous(expand = c(0, 0))

png(file.path(base_dir, "figures/EggaS_index.png"), width = pagewidth, height = pagewidth*0.7, units = "mm", res = 300)
print(p)
dev.off()


# Barents Sea Ecosystem Survey (BESS) index excluding the juvenile indices below ####

message("BESS indices")

EcoS_SI_as_biomass_index <- TRUE # Switch to shift between abundance and biomass indices

### Combined ####

EcoS_SI <- lapply(BESS, function(k) k$station_length) %>%
  bind_rows() %>%
  filter(
    !grepl("^O|^X", Stratum),
    IndividualTotalLength > 27,
    #IndividualTotalLength < 65
  )

removals <-
  '!((year == "2005" & HaulKey == "3815") | (year == "2022" & HaulKey == "2446"))'

tmp <-
  EcoS_SI %>%
  {if(EcoS_SI_as_biomass_index) rename(., value = Biomass) else rename(., value = Abundance)} %>%
  mutate(year = as.integer(year)) %>%
  group_by(year, HaulKey) %>%
  summarise(value = sum(value)/1e6) %>%
  mutate(keep = eval(rlang::parse_expr(removals))) # %>%
  # mutate(year = year - 1)

po <-
  ggplot(data = tmp %>% filter(keep), aes(x = year, y = value)) +
  geom_smooth(method = 'loess', formula = 'y ~ x', span = 0.5) +
  geom_boxplot(aes(group = year), alpha = 0.5) +
  geom_point(data = tmp %>% filter(!keep), aes(x = year, y = value),
             color = "red", alpha = 0.7) +
  labs(
    y = ifelse(EcoS_SI_as_biomass_index,
               "Biomass contribution per station (kt)",
               "Abundance contribution per station (millions)"),
    x = "Year") +
  scale_x_continuous(expand = c(0, 0), breaks = seq(1900, 2030, 2)) +
  scale_y_continuous(expand = expansion(mult = c(0, 0.01))) +
  expand_limits(y = 0)

pol <-
  ggplot(data = tmp %>% filter(keep), aes(x = year, y = value)) +
  geom_smooth(method = 'loess', formula = 'y ~ x', span = 0.5) +
  geom_boxplot(aes(group = year), alpha = 0.5) +
  geom_point(data = tmp %>% filter(!keep), aes(x = year, y = value),
             color = "red", alpha = 0.7) +
  labs(
    y = ifelse(EcoS_SI_as_biomass_index,
               "Biomass contribution per station (kt)",
               "Abundance contribution per station (millions)"),
    x = "Year") +
  scale_x_continuous(expand = c(0, 0), breaks = seq(1900, 2030, 2)) +
  scale_y_log10(expand = expansion(mult = c(0, 0.01)))

EcoS_SI <- EcoS_SI %>%
  filter(!!!rlang::parse_exprs(removals)) %>%
  group_by(year) %>%
  {if(EcoS_SI_as_biomass_index) summarise(., weight = sum(Biomass, na.rm = TRUE)) else summarise(., number = sum(Abundance, na.rm = TRUE))} %>%
  dplyr::mutate(step = "1", length = "len28", .after = "year") %>%
  add_g3_attributes(
    list(
      length = mfdb_interval("len", c(28, 65)),
      timestep = model_params$timestep_fun),
    warn = 2
  )

png(file.path(base_dir, "figures/EcoS_outliers.png"), width = pagewidth, height = pagewidth*1.2, units = "mm", res = 300)
print(cowplot::plot_grid(
  cowplot::plot_grid(po + ggtitle("Per station"), pol + ggtitle("Per station, log scale"), ncol = 2),
  ggplot(
    EcoS_SI %>%
      mutate(value = if(EcoS_SI_as_biomass_index) weight else number),
    aes(x = year, y = value/1e6)
  ) +
    geom_col() +
    labs(
      y = ifelse(EcoS_SI_as_biomass_index,
                 "Survey index biomass (kt)",
                 "Survey index abundance (millions)"),
      x = "Year") +
    scale_x_continuous(expand = c(0, 0), breaks = seq(1900, 2030, 2)) +
    scale_y_continuous(expand = c(0, 0)) +
    ggtitle("Index"),
  ncol = 1
))
dev.off()

#### Plot ####

p <- ggplot(
  EcoS_SI %>%
    mutate(value = if(EcoS_SI_as_biomass_index) weight else number),
  aes(x = year, y = value/1e6)
) +
  geom_col() +
  labs(
    y = ifelse(EcoS_SI_as_biomass_index,
               "Survey index biomass (kt)",
               "Survey index abundance (millions)"),
    x = "Year") +
  scale_x_continuous(expand = c(0, 0), breaks = seq(1900, 2030, 2)) +
  scale_y_continuous(expand = c(0, 0))

png(file.path(base_dir, "figures/EcoS_index.png"), width = pagewidth, height = pagewidth*0.7, units = "mm", res = 300)
print(p)
dev.off()

#### Plot with confidence intervals ####

if(EcoS_SI_as_biomass_index) {
  if(all(sapply(BESS, function(k) "bs_biomass" %in% names(k)))) {
    bs_EcoS_SI <- lapply(BESS, function(k) k$bs_biomass) %>%
      bind_rows() %>%
      filter(
        !grepl("^O|^X", Stratum),
        IndividualTotalLength > 27
      )
  } else {
    bs_EcoS_SI <- NULL
  }
} else {
  if(all(sapply(BESS, function(k) "bs_abundance" %in% names(k)))) {
    bs_EcoS_SI <- lapply(BESS, function(k) k$bs_abundance) %>%
      bind_rows() %>%
      filter(
        !grepl("^O|^X", Stratum),
        IndividualTotalLength > 27
      )
  } else {
    bs_EcoS_SI <- NULL
  }
}

if(!is.null(bs_EcoS_SI)) {
  p <- bs_EcoS_SI %>%
    {if(EcoS_SI_as_biomass_index) {
      rename(., value = Biomass_sum_mean,
             upr = `Biomass_sum_97.5%`,
             lwr = `Biomass_sum_2.5%`)
    } else {
      rename(., value = Abundance_sum_mean,
             upr = `Abundance_sum_97.5%`,
             lwr = `Abundance_sum_2.5%`)
    }} %>%
    mutate(year = as.integer(year)) %>%
    group_by(year) %>%
    summarise(value = sum(value, na.rm = TRUE)/1e6,
              upr = sum(upr, na.rm = TRUE)/1e6,
              lwr = sum(lwr, na.rm = TRUE)/1e6) %>%
    ggplot(aes(x = year, y = value, ymin = upr, ymax = lwr)) +
    geom_ribbon(alpha = 0.3, color = NA) +
    geom_path() +
    geom_point(shape = 21) +
    geom_path(
      data = EcoS_SI %>%
        mutate(value = if(EcoS_SI_as_biomass_index) weight else number),
      aes(x = year, y = value/1e6),
      linetype = 2, inherit.aes = FALSE
    ) +
    labs(
      y = ifelse(EcoS_SI_as_biomass_index,
                 "Survey index biomass ± 95% CIs (kt)",
                 "Survey index abundance ± 95% CIs (millions)"),
      x = "Year") +
    expand_limits(y = 0) +
    scale_x_continuous(expand = c(0, 0), breaks = seq(1900, 2030, 2)) +
    scale_y_continuous(expand = c(0, 0)) +
    coord_cartesian(clip = "off")

  png(file.path(base_dir, "figures/EcoS_index_with_uncertainty.png"), width = pagewidth, height = pagewidth*0.7, units = "mm", res = 300)
  print(p)
  dev.off()
}

## Juvenile indices from Barents Sea Ecosystem Survey data (BESS) ####

### Juv 1 ####

Juv_SI_1 <- lapply(BESS, function(k) k$station_length) %>%
  bind_rows() %>%
  filter(
    !grepl("^O|^X", Stratum),
    IndividualTotalLength <= 17,
    year %in% model_params$year_range[model_params$year_range != "2014"]
  )

po <- Juv_SI_1 %>%
  rename(., value = Abundance) %>%
  # mutate(year = year - 1) %>%
  group_by(year, HaulKey) %>%
  summarise(value = sum(value)) %>%
  ggplot(aes(x = year, y = value/1e6)) +
  geom_smooth(method = 'loess', formula = 'y ~ x', span = 0.5) +
  geom_boxplot(aes(group = year), alpha = 0.5) +
  labs(
    y = "Abundance contribution per station (millions)",
    x = "Year") +
  scale_x_continuous(expand = c(0, 0), breaks = seq(1900, 2030, 2)) +
  scale_y_continuous(expand = expansion(mult = c(0, 0.01))) +
  expand_limits(y = 0)

pol <- Juv_SI_1 %>%
  rename(., value = Abundance) %>%
  # mutate(year = year - 1) %>%
  group_by(year, HaulKey) %>%
  summarise(value = sum(value)) %>%
  ggplot(aes(x = year, y = value/1e6)) +
  geom_smooth(method = 'loess', formula = 'y ~ x', span = 0.4) +
  geom_boxplot(aes(group = year), alpha = 0.5) +
  labs(
    y = NULL,
    x = "Year") +
  scale_x_continuous(expand = c(0, 0), breaks = seq(1900, 2030, 2)) +
  scale_y_log10(expand = expansion(mult = c(0, 0.01)))

Juv_SI_1 <- Juv_SI_1 %>%
  group_by(year) %>%
  summarise(., number = sum(Abundance, na.rm = TRUE)) %>%
  dplyr::mutate(step = "1", length = "len9", .after = "year") %>%
  add_g3_attributes(
    list(
      length = mfdb_interval("len", c(9,17) #, open_ended = c("lower")
                             ),
      timestep = model_params$timestep_fun),
    warn = 2
  )

# Juv_SI_1$year <- Juv_SI_1$year - 1
#
# attributes(Juv_SI_1)$year <- stats::setNames(
#   lapply(unique(Juv_SI_1$year), function(k) k),
#   lapply(unique(Juv_SI_1$year), function(k) k)
# )

png(file.path(base_dir, "figures/Juv_SI_1_outliers.png"), width = pagewidth, height = pagewidth*1.2, units = "mm", res = 300)
print(cowplot::plot_grid(
  cowplot::plot_grid(po + ggtitle("Per station"), pol + ggtitle("Per station, log scale"), ncol = 2),
  Juv_SI_1 %>%
    ggplot(., aes(x = year, y = number/1e6)) +
    geom_col() +
    labs(y = "Abundance (millions)", x = "Year") +
    scale_x_continuous(expand = c(0, 0), breaks = seq(1900, 2030, 2)) +
    scale_y_continuous(expand = c(0, 0)) +
    ggtitle("Index"),
  ncol = 1
))
dev.off()

### Juv 2 ####

Juv_SI_2 <- lapply(BESS, function(k) k$station_length) %>%
  bind_rows() %>%
  filter(
    !grepl("^O|^X", Stratum),
    IndividualTotalLength <= 27,
    IndividualTotalLength >= 18,
    year %in% model_params$year_range[model_params$year_range != "2014"]
  )

removals <-
  '!((year == "2022" & HaulKey == "2446") | (year == "2006" & HaulKey == "3421"))' # (year == "2020" & HaulKey == "3253"))

tmp <-
  Juv_SI_2 %>%
  rename(., value = Abundance) %>%
  mutate(year = as.integer(year)) %>%
  group_by(year, HaulKey) %>%
  summarise(value = sum(value)/1e6) %>%
  mutate(keep = eval(rlang::parse_expr(removals))) # %>%
  # mutate(year = year - 1)

po <- ggplot(data = tmp %>% filter(keep), aes(x = year, y = value)) +
  geom_smooth(method = 'loess', formula = 'y ~ x', span = 0.5) +
  geom_boxplot(aes(group = year), alpha = 0.5) +
  geom_point(data = tmp %>% filter(!keep), aes(x = year, y = value), color = "red", alpha = 0.7) +
  labs(
    y = "Abundance contribution per station (millions)",
    x = "Year") +
  scale_x_continuous(expand = c(0, 0), breaks = seq(1900, 2030, 2)) +
  scale_y_continuous(expand = expansion(mult = c(0, 0.01))) +
  expand_limits(y = 0)

pol <-
  ggplot(data = tmp %>% filter(keep), aes(x = year, y = value)) +
  geom_smooth(method = 'loess', formula = 'y ~ x', span = 0.5) +
  geom_boxplot(aes(group = year), alpha = 0.5) +
  geom_point(data = tmp %>% filter(!keep), aes(x = year, y = value), color = "red", alpha = 0.7) +
  labs(
    y = "Abundance contribution per station (millions)",
    x = "Year") +
  scale_x_continuous(expand = c(0, 0), breaks = seq(1900, 2030, 2)) +
  scale_y_log10(expand = expansion(mult = c(0, 0.01)))

Juv_SI_2 <- Juv_SI_2 %>%
  filter(!!!rlang::parse_exprs(removals)) %>%
  group_by(year) %>%
  summarise(., number = sum(Abundance, na.rm = TRUE)) %>%
  dplyr::mutate(step = "1", length = "len18", .after = "year") %>%
  add_g3_attributes(
    list(
      length = mfdb_interval("len", c(18,27)),
      timestep = model_params$timestep_fun),
    warn = 2
  ) # %>%
  # mutate(year = year - 1)

# attributes(Juv_SI_2)$year <- stats::setNames(
#   lapply(unique(Juv_SI_2$year), function(k) k),
#   lapply(unique(Juv_SI_2$year), function(k) k)
# )

png(file.path(base_dir, "figures/Juv_SI_2_outliers.png"), width = pagewidth, height = pagewidth*1.2, units = "mm", res = 300)
print(cowplot::plot_grid(
  cowplot::plot_grid(po + ggtitle("Per station"), pol + ggtitle("Per station, log scale"), ncol = 2),
  Juv_SI_2 %>%
    ggplot(., aes(x = year, y = number/1e6)) +
    geom_col() +
    labs(y = "Abundance (millions)", x = "Year") +
    scale_x_continuous(expand = c(0, 0), breaks = seq(1900, 2030, 2)) +
    scale_y_continuous(expand = c(0, 0)) +
    ggtitle("Index"),
  ncol = 1
))
dev.off()

### Plot ####

p <- bind_rows(
  Juv_SI_1 %>% mutate(length = "Juv_SI_1 (10-17 cm)"),
  Juv_SI_2 %>% mutate(length = "Juv_SI_2 (18-27 cm)")
) %>%
  ggplot(., aes(x = year, y = number/1e6)) +
  geom_col() +
  facet_wrap(~length, scales = "free_y", ncol = 1) +
  labs(y = "Abundance (millions)", x = "Year") +
  scale_x_continuous(expand = c(0, 0), breaks = seq(1900, 2030, 2)) +
  scale_y_continuous(expand = c(0, 0))

png(file.path(base_dir, "figures/Juvenile_indices.png"), width = pagewidth, height = pagewidth*0.7, units = "mm", res = 300)
print(p)
dev.off()

#### Plot with confidence intervals ####

if(all(sapply(BESS, function(k) "bs_abundance" %in% names(k)))) {
  bs_Juv_SI <- lapply(BESS, function(k) k$bs_abundance) %>%
    bind_rows() %>%
    filter(
      !grepl("^O|^X", Stratum),
      IndividualTotalLength < 28,
      IndividualTotalLength >= 10
      # !year %in% 2014
    )
} else {
  bs_Juv_SI <- NULL
}


if(!is.null(bs_Juv_SI)) {
  p <- bs_Juv_SI %>%
    mutate(
      cohort = cut(IndividualTotalLength, breaks = c(0,18,28), right = FALSE,
                   labels = c("Juv_SI_1 (10-17 cm)", "Juv_SI_2 (18-27 cm)"))
    ) %>%
    rename(., value = Abundance_sum_mean,
           upr = `Abundance_sum_97.5%`,
           lwr = `Abundance_sum_2.5%`) %>%
    mutate(year = as.integer(year)) %>%
    group_by(year, cohort) %>%
    reframe(value = sum(value, na.rm = TRUE)/1e6,
            upr = sum(upr, na.rm = TRUE)/1e6,
            lwr = sum(lwr, na.rm = TRUE)/1e6) %>%
    ggplot(aes(x = year, y = value, ymin = upr, ymax = lwr, color = cohort,
               fill = cohort)) +
    geom_ribbon(alpha = 0.3, color = NA) +
    geom_path() +
    geom_point(shape = 1) +
    geom_path(
      data = bind_rows(
        Juv_SI_1 %>% mutate(length = "Juv_SI_1 (10-17 cm)"),
        Juv_SI_2 %>% mutate(length = "Juv_SI_2 (18-27 cm)")
      ) %>%
        mutate(value = number #,
               #year = year + 1
               ),
      aes(x = year, y = value/1e6, color = length),
      linetype = 2, inherit.aes = FALSE
    ) +
    labs(
      y = "Abundance ± 95% CIs (millions)",
      x = "Year",
      color = "Cohort",
      fill = "Cohort") +
    expand_limits(y = 0) +
    scale_x_continuous(expand = c(0, 0), breaks = seq(1900, 2030, 2)) +
    scale_y_continuous(expand = c(0, 0)) +
    coord_cartesian(clip = "off") +
    theme(legend.position = "bottom")

  png(file.path(base_dir, "figures/Juvenile_indices_with_uncertainty.png"), width = pagewidth, height = pagewidth*0.7, units = "mm", res = 300)
  print(p)
  dev.off()
}

# Winter survey index ####

# WinterS_SI <- read.csv('../ghl-gadget-data/data/out/Winter survey index.csv') %>%
#   dplyr::select(startyear, bm) %>%
#   rename("year" = "startyear", "weight" = "bm") %>%
#   add_g3_attributes(
#     params = list(
#       year = model_params$year_range,
#       step = model_params$timestep_fun,
#       length = mfdb_interval(
#         "all", c(40, stock_params$maxlength),
#         open_ended = c("upper"))
#     )
#   )
#
# p <- ggplot(WinterS_SI, aes(x = year, y = weight)) +
#   geom_col() +
#   labs(y = "Survey index biomass (1000 t)", x = "Year") +
#   scale_x_continuous(expand = c(0, 0), breaks = seq(1900, 2030, 2)) +
#   scale_y_continuous(expand = c(0, 0))
#
# ggsave(filename = file.path(base_dir, "figures/WinterS_index.png"),
#        plot = print(p), width = pagewidth, height = pagewidth*0.7,
#        units = "mm", bg = "white")
#
# rm(p)

# Russian survey index ####

message("Russian indices")

RussianS_SI_as_biomass_index <- FALSE # Switch to shift between abundance and biomass indices

RussianS_SI_female <-
  readxl::read_excel("data/Russian survey index data.xlsx", "F slope") %>%
  filter(!is.na(`20.0-20.9`)) %>%
  {if(RussianS_SI_as_biomass_index) mutate(., weight = biomass*1e3) else mutate(., number = abundance*1e3)} %>%
  {if(RussianS_SI_as_biomass_index) dplyr::select(., c(year, weight)) else dplyr::select(., c(year, number))} %>%
  dplyr::mutate(step = "1", length = "len28", .after = "year") %>%
  add_g3_attributes(
    list(length = mfdb_interval(
      "len", c(28, 108),
      open_ended = c("upper")),
      timestep = model_params$timestep_fun),
    warn = 2
  )

RussianS_SI_male <-
  readxl::read_excel("data/Russian survey index data.xlsx", "M slope") %>%
  filter(!is.na(`20.0-20.9`)) %>%
  {if(RussianS_SI_as_biomass_index) mutate(., weight = biomass*1e3) else mutate(., number = abundance*1e3)} %>%
  {if(RussianS_SI_as_biomass_index) dplyr::select(., c(year, weight)) else dplyr::select(., c(year, number))} %>%
  dplyr::mutate(step = "1", length = "len28", .after = "year") %>%
  add_g3_attributes(
    list(length = mfdb_interval(
      "len", c(28, 108),
      open_ended = c("upper")),
      timestep = model_params$timestep_fun),
    warn = 2
  )

RussianS_SI <-
  bind_rows(RussianS_SI_female, RussianS_SI_male) %>%
  group_by(year, step, length) %>%
  {if(RussianS_SI_as_biomass_index) summarise(., weight = sum(weight)) else summarise(., number = sum(number))} %>%
  ungroup() %>%
  add_g3_attributes(
    list(length = mfdb_interval(
      "len", c(28, 108),
      open_ended = c("upper")),
      timestep = model_params$timestep_fun),
    warn = 2
  )

p <- bind_rows(RussianS_SI_female %>% mutate(sex = "F"),
               RussianS_SI_male %>% mutate(sex = "M")
) %>%
  mutate(value = if(RussianS_SI_as_biomass_index) weight else number) %>%
  ggplot(., aes(x = year, y = value/1e6, fill = sex)) +
  geom_col(position = position_stack(reverse = TRUE)) +
  scale_fill_manual("Fill", values = c("F" = "#FF5F68", "M" = "#449BCF")) +
  scale_x_continuous("Year", expand = c(0, 0), breaks = seq(1900, 2030, 2)) +
  scale_y_continuous(
    if(RussianS_SI_as_biomass_index) "Survey index biomass (kt)" else {
      "Survey index abundance (millions)"
    },
    expand = c(0, 0)) +
  theme(legend.position = "bottom")

ggsave(filename = file.path(base_dir, "figures/RussianS_index.png"),
       plot = print(p), width = pagewidth, height = pagewidth*0.7,
       units = "mm", bg = "white")

# Russian CPUE index ####

Rus_CPUE_SI <- read.csv('data/Russian CPUE index data.csv') %>%
  rename("weight" = "cpue") %>%
  filter(year %in% model_params$year_range[model_params$year_range > 1980]) %>%
  dplyr::mutate(step = "1", length = "len40", .after = "year") %>%
  add_g3_attributes(
    params = list(
      length = mfdb_interval(
        "len", c(40, stock_params$maxlength),
        open_ended = c("upper"))
    ),
    warn = 2
  )

p <- ggplot(Rus_CPUE_SI, aes(x = year, y = weight)) +
  geom_col() +
  labs(y = "Survey index biomass (1000 t)", x = "Year") +
  scale_x_continuous(expand = c(0, 0), breaks = seq(1900, 2030, 2)) +
  scale_y_continuous(expand = c(0, 0))

ggsave(filename = file.path(base_dir, "figures/Russian_CPUE_index.png"),
       plot = print(p), width = pagewidth, height = pagewidth*0.7,
       units = "mm", bg = "white")

rm(p)

# Save ####

message("Saving and ploting")

save(EggaN_SI_female, EggaN_SI_male, EggaN_SI,
     EggaS_SI_female, EggaS_SI_male, EggaS_SI,
     EcoS_SI, Juv_SI_1, Juv_SI_2,
     RussianS_SI_female, RussianS_SI_male, RussianS_SI, Rus_CPUE_SI,
     file = file.path(base_dir, "data/Survey indices to Gadget.rda"))

# Plot comparisons ####

## All indices ####

p <- dplyr::bind_rows(
  EggaN_SI_female %>%
    mutate(value = if(EggaN_SI_as_biomass_index) weight else number) %>%
    dplyr::mutate(index = "EggaN_SI_female",
                  p = value/max(value)) %>%
    dplyr::select(year, step, length, index, p),
  EggaN_SI_male %>%
    mutate(value = if(EggaN_SI_as_biomass_index) weight else number) %>%
    dplyr::mutate(index = "EggaN_SI_male",
                  p = value/max(value)) %>%
    dplyr::select(year, step, length, index, p),
  EggaS_SI %>%
    mutate(value = number) %>%
    dplyr::mutate(index = "EggaS_SI",
                  p = value/max(value)) %>%
    dplyr::select(year, step, length, index, p),
  Juv_SI_1 %>%
    dplyr::mutate(index = "Juv_SI_1",
                  p = number/max(number)) %>%
    dplyr::select(-number),
  Juv_SI_2 %>%
    dplyr::mutate(index = "Juv_SI_2",
                  p = number/max(number)) %>%
    dplyr::select(-number),
  EcoS_SI %>%
    mutate(value = if(EcoS_SI_as_biomass_index) weight else number) %>%
    dplyr::mutate(index = "EcoS_SI",
                  p = value/max(value)) %>%
    dplyr::select(year, step, length, index, p),
  RussianS_SI_female %>%
    mutate(value = if(RussianS_SI_as_biomass_index) weight else number) %>%
    dplyr::mutate(index = "RussianS_SI_female",
                  p = value/max(value)) %>%
    dplyr::select(year, step, length, index, p),
  RussianS_SI_male %>%
    mutate(value = if(RussianS_SI_as_biomass_index) weight else number) %>%
    dplyr::mutate(index = "RussianS_SI_male",
                  p = value/max(value)) %>%
    dplyr::select(year, step, length, index, p),
  Rus_CPUE_SI %>%
    dplyr::mutate(index = "Rus_CPUE_SI",
                  p = weight/max(weight)) %>%
    dplyr::select(-weight)
) %>%
  ggplot() +
  geom_smooth(aes(x = year, y = p), method = 'loess', formula = 'y ~ x') +
  geom_line(aes(x = year, y = p, color = index)) +
  labs(x = "Year", y = "Standardized index", color = "Survey index") +
  scale_x_continuous(expand = c(0, 0), breaks = seq(1900, 2030, 2)) +
  scale_y_continuous(expand = c(0, 0)) +
  theme(legend.position = "bottom")

ggsave(
  filename = file.path(base_dir, "figures/Survey_index_comparison.png"),
  plot = print(p), width = pagewidth, height = pagewidth*0.7,  units = "mm",
  bg = "white")

## Adult indices ####

p <- dplyr::bind_rows(
  EggaN_SI %>%
    mutate(value = if(EggaN_SI_as_biomass_index) weight else number) %>%
    dplyr::mutate(index = "EggaN_SI",
                  p = value/max(value)) %>%
    dplyr::select(year, step, length, index, p),
  EggaS_SI %>%
    mutate(value = number) %>%
    dplyr::mutate(index = "EggaS_SI",
                  p = value/max(value)) %>%
    dplyr::select(year, step, length, index, p),
  EcoS_SI %>%
    mutate(value = if(EcoS_SI_as_biomass_index) weight else number) %>%
    dplyr::mutate(index = "EcoS_SI",
                  p = value/max(value)) %>%
    dplyr::select(year, step, length, index, p),
  RussianS_SI %>%
    mutate(value = if(RussianS_SI_as_biomass_index) weight else number) %>%
    dplyr::mutate(index = "RussianS_SI",
                  p = value/max(value)) %>%
    dplyr::select(year, step, length, index, p),
  Rus_CPUE_SI %>%
    dplyr::mutate(index = "Rus_CPUE_SI",
                  p = weight/max(weight)) %>%
    dplyr::select(-weight)
) %>%
  ggplot() +
  geom_smooth(aes(x = year, y = p), method = 'loess', formula = 'y ~ x') +
  geom_line(aes(x = year, y = p, color = index)) +
  labs(x = "Year", y = "Standardized index", color = "Survey index") +
  scale_x_continuous(expand = c(0, 0), breaks = seq(1900, 2030, 2)) +
  scale_y_continuous(expand = c(0, 0)) +
  theme(legend.position = "bottom")

ggsave(
  filename = file.path(base_dir, "figures/Adult_survey_index_comparison.png"),
  plot = print(p), width = pagewidth, height = pagewidth*0.7,  units = "mm",
  bg = "white")

message("Done compiling survey indices")
