## ---------------------------
##
## Script name: Download length, sex, and substock (maturity) distribution data
##              into the data folder
##
## ---------------------------

## Setup ####

message("Setting up database...")

### Source or list custom functions used within the script ####

split_fun <- function(x) {
  mean(
    c(as.numeric(gsub("[^0-9.-]", "", sapply(strsplit(as.character(x), ","), "[", 1))),
      as.numeric(gsub("[^0-9.-]", "", sapply(strsplit(as.character(x), ","), "[", 2))))
  )
}

### Load data ####

load("data/Ghl strata polygons.rda")
load("data/gearList.rda")

### Sampling types (MFDB artifact) ####

tmp <- c("ReferanseflatenHav", "ReferanseflatenKyst", "Forskningsfartoy",
         "Leiefartoy", "OvervakningstjenestenKommersieltFiske", "Kystvakt",
         "Provebat", "Moreforskning", "OvervakningstjenestenInnleidFartoy",
         "Landings", "Winter Survey", "Ecosystem Survey", "EggaN Survey",
         "EggaS Survey")

SamplingTypeList <- data.frame(
  id = c(2:5, 9:13, 99, 105, 106, 1016, 1025),
  name = toupper(abbreviate(tmp, minlength = 3, named = FALSE)),
  description = tmp
)

rm(tmp)

### Create the data objects ####

stnall <- dplyr::tbl(con_db, "stnall")
indall <- dplyr::tbl(con_db, "indall")
ageall <- dplyr::tbl(con_db, "ageall")
mission <- dplyr::tbl(con_db, "mission")
updated <- dplyr::tbl(con_db, "metadata") %>% dplyr::pull(timestart) %>% as.Date()
csindex <- dplyr::tbl(con_db, "csindex")

### Cruise series filtering ####

if(exists("csindex")) {
  csList <- csindex %>%
    collect() %>%
    dplyr::select(cruiseseriescode, name) %>%
    unique() %>%
    arrange(cruiseseriescode)
} else {
  csList <- BioticExplorerServer::cruiseSeries %>%
    dplyr::select(cruiseseriescode, name) %>%
    unique() %>%
    arrange(cruiseseriescode)
}

selCS <- csList[grepl("winter|ecosystem cruise in autumn|continental", csList$name, ignore.case = TRUE),]
csFilt <- selCS$cruiseseriescode

#### Filter ####

# filtExp <- paste(sapply(csFilt, function(k) {paste0("cruiseseriescode %like% '", k, ",%' | cruiseseriescode %like% '%,", k, "' | cruiseseriescode %like% '%,", k, ",%'", " | cruiseseriescode %in% c('", k,"')")}), collapse = " | ")

sel.cols <- c("missiontype", "missionnumber", "cruiseseriescode", "startyear", "platform", "serialnumber", "catchpartnumber", "specimenid", "platformname", "cruise", "missiontypename", "gear", "nation", "catchplatform", "stationstartdate", "stationstopdate", "stationtype", "latitudestart", "longitudestart", "area", "location", "bottomdepthstart", "fishingdepthstart", "gearno", "sweeplength", "gearcount", "distance", "gearcondition", "samplequality", "icesarea", "gearname", "gearcategory", "commonname", "catchcategory", "catchweight", "lengthsampleweight", "catchcount", "lengthsamplecount", "sampletype", "length", "sex", "maturationstage", "specialstage", "individualweight", "age", "readingdate", "agereader", "readability") # Select useful columns to save processing power

###################################### #
# Individual data from the database ####

message("Acquiring individual data...")

x <- indall %>%
  dplyr::filter(
    commonname == "blåkveite",
    !is.na(length)
  ) %>%
  dplyr::select(all_of(sel.cols)) %>%
  collect()

message("Processing individual data...")

x$maturationstage[x$nation %in% c(140, 90)] <- NA # Remove Russian maturity readings (they are wrong in some cases)
x$specialstage[x$nation %in% c(140, 90)] <- NA # Remove Russian maturity readings (they are wrong in some cases)

x$id <- paste(x$cruiseseriescode, x$startyear, x$platformname, x$serialnumber,
              sep = "_")

### Remove stations with unclarities in catchcount / weight ####
tmp <- x %>%
  dplyr::filter(
    cruiseseriescode %in% csFilt,
    gearcategory == "Bottom trawls",
    samplequality == "1",
    gearcondition %in% 1:2,
    !stationtype %in% c("C", "2", "13")
  ) %>%
  group_by(id) %>%
  reframe(catchweight = sum(catchweight, na.rm = TRUE),
          catchcount = sum(catchcount, na.rm = TRUE),
          lengthsampleweight = sum(lengthsampleweight, na.rm = TRUE),
          lengthsamplecount = sum(lengthsamplecount, na.rm = TRUE)
  ) %>%
  filter(catchcount < lengthsamplecount | catchweight < lengthsampleweight) %>%
  pull(id)

x <- x %>%
  filter(!id %in% tmp) %>%
  dplyr::select(-id)

rm(tmp)

### Assign the age reading method ####

x$readingtype <-
  if_else(is.na(x$age), NA,
         if_else(x$agereader %in% c("KJGT","TOEL","KRWD","ROLA"), "new_qualified_reader",
                if_else(!is.na(x$agereader), "new_other_reader",
                       if_else(!is.na(x$readingdate), "new_unknown_reader",
                              if_else(x$startyear >= 2007, "new_no_readingdate",
                                     if_else(x$startyear < 2000, "old", "in_between"
                                     ))))))

### Correct maturation stage ####

matStages <- readxl::read_excel("data/Greenland halibut maturity stages.xlsx")
ms <- na.omit(matStages[c("maturationstage", "maturationstage_becomes")])
ms[,grep("becomes", names(ms))] <-
  suppressWarnings(as.integer(ms[[grep("becomes", names(ms))]]))
ss <- na.omit(matStages[c("specialstage", "specialstage_becomes")])
ss[,grep("becomes", names(ss))] <-
  suppressWarnings(as.integer(ss[[grep("becomes", names(ss))]]))

x <- x %>% mutate(
  maturationstage =
    recode(maturationstage, !!!setNames(ms$maturationstage_becomes,
                                        ms$maturationstage)),
  specialstage = if_else(specialstage > 7, NA, specialstage),
  specialstage =
    recode(specialstage, !!!setNames(ss$specialstage_becomes,
                                     ss$specialstage)))

x[x$maturationstage %in% "1" & is.na(x$specialstage), ]$specialstage <- 1

x[x$sex %in% "1" & !is.na(x$specialstage),]$maturationstage <-
  x[x$sex %in% "1" & !is.na(x$specialstage),]$specialstage

### Save 'raw' data ####

message("Saving raw data...")

saveRDS(x, "data/Length and age raw data.rds", compress = "xz")

################# #
# Survey data ####

message("Survey data")

## Modify data for gadget ####

### Length data for all surveys ###

message("Processing length data...")

data(fdir_areas, package = "ggOceanMaps")

### Length and age data for EggaN, EggaS, and EcoS ####
#### The number of fish in each length is extrapolated to the entire catch
#### by first estimating the number of fish in catch (divide catch weight by
#### length sample weight). Then, the estimated catch count is divided by
#### length sample count. The sum of these counts matches the catch count
#### This gives number of super-individuals for each
#### length measured fish. These length measured fish are repeated (?uncount)
#### in the dataset to estimate the total number of fish in a trawl haul.
#### The use of super-inviduals can be turned on and off by specifying
#### uncount(superindividuals) in the following data processing steps.

ldist_surv <- x %>%
  dplyr::filter(
    cruiseseriescode %in% csFilt,
    gearcategory == "Bottom trawls",
    samplequality == "1",
    gearcondition %in% 1:2,
    !stationtype %in% c("C", "2", "13")
  ) %>%
  filter(!is.na(latitudestart), !is.na(longitudestart)) %>%
  sf::st_as_sf(coords = c("longitudestart", "latitudestart"), crs = 4326,
               remove = FALSE) %>%
  rownames_to_column("id") %>%
  sf::st_join(strata$strata %>%
                mutate(population = paste0(geostrata.name, interval))) %>%
  filter(!duplicated(id)) %>%
  sf::st_join(fdir_main_areas %>% mutate(areacell = as.integer(main_area))) %>%
  filter(!duplicated(id)) %>%
  sf::st_set_geometry(NULL) %>%
  mutate(cruiseseriescode = recode(cruiseseriescode, "20,6" = "6")) %>%
  mutate(cruiseseriescode = paste0("10", cruiseseriescode)) %>%
  mutate(
    sampling_type =
      recode(cruiseseriescode,
             !!!setNames(SamplingTypeList$name, SamplingTypeList$id)),
    gear = recode(gear, !!!setNames(gearList$gearcategory, gearList$code)),
    month = ifelse(is.na(stationstartdate),
                   lubridate::month(stationstopdate),
                   lubridate::month(stationstartdate)),
    sex = ifelse(sex > 2, NA, ifelse(sex == 1, "F", "M")),
    length = 100*length + 1e-6) %>% # addition to avoid a floating point issue
  replace_na(list(population = "Outside")) %>%
  filter(!is.na(month)) %>%
  mutate( # part to 'raise' to number of fish in catch
    # mean_weight = catchweight/catchcount,
    catchcount2 = if_else(
      catchweight > lengthsampleweight, # make catchcount consistent
      round((catchweight/lengthsampleweight)*lengthsamplecount),
      catchcount)
  ) %>%
  mutate(catchcount = if_else(!is.na(catchcount2), catchcount2, catchcount)) %>%
  dplyr::select(-catchcount2) %>%
  mutate(repN = round(catchcount/lengthsamplecount, 0),
         repN = ifelse(is.na(repN) | repN == 0, lengthsamplecount, repN)) %>%
  rename("superindividuals" = "repN")

### Age data for surveys ####
#### The number of fish at age and length is estimated as follows: first,
#### surveys containing new age reading data are subsetted. Then total number
#### of fish in each length & sex bin in each trawl haul is estimated using
#### super-individuals. Next, the proportion of ages per length group is
#### estimated for each year and CRUISE SERIES (sampling_type). This proportion
#### is finally allocated to the length data on station level. Consequently,
#### the sum of est_n matches the sum of length-binned super-individual data
#### on year + cruise series level for surveys that contained age data.

message("Processing age data...")

age_det_surveys <- ldist_surv %>%
  dplyr::filter(!is.na(age)) %>%
  dplyr::filter(grepl("new", readingtype)) %>%
  mutate(cond = paste(sampling_type, cruise, sep = "_")) %>%
  pull(cond) %>% table

aldist_surv <- ldist_surv %>%
  mutate(cond = paste(sampling_type, cruise, sep = "_")) %>%
  filter(cond %in% names(age_det_surveys)) %>%
  replace_na(list(sex = "U")) %>%
  mutate(length_bin = cut(length, seq(0,115,5)))

lbin_age_proportions <-
  aldist_surv %>%
  filter(!is.na(age),
         grepl("new", readingtype)) %>%
  # uncount(superindividuals) %>%
  group_by(gear, sampling_type, startyear, cruise, sex, age,
           length_bin) %>%
  count() %>%
  group_by(gear, sampling_type, startyear, cruise, sex,
           length_bin) %>%
  mutate(aged = sum(n), pr = n/aged) %>%
  arrange(sampling_type, startyear, cruise, sex, length_bin, age)

aldist_surv <-
  aldist_surv %>%
  uncount(superindividuals) %>%
  group_by(gear, sampling_type, startyear, month, areacell, population, cruise,
           serialnumber, sex, length_bin) %>%
  reframe(total = n()) %>%
  arrange(sampling_type, startyear, cruise, serialnumber, sex, length_bin) %>%
  left_join(lbin_age_proportions,
            multiple = "all",
            by = join_by(gear, sampling_type, startyear, cruise, sex, length_bin),
            relationship = "many-to-many") %>%
  mutate(est_n = ifelse(is.na(pr), total, pr * total)) %>%
  dplyr::select(-aged, -pr, -n) %>%
  rename(year = startyear) %>%
  relocate(age, .after = "length_bin") %>%
  mutate(length = sapply(length_bin, split_fun), .before = length_bin) # this line takes 20 s

attributes(aldist_surv)$updated <- updated

message("Saving age data...")

saveRDS(aldist_surv, file = "data/Age-length data for surveys.rds", compress = "xz")
rm(lbin_age_proportions)

### Clean length data ###

message("Processing length data...")

ldist_surv <- ldist_surv %>%
  rename(year = startyear, weight = individualweight, maturity_stage = maturationstage) %>%
  select(gear, sampling_type, year, month, areacell, population, age,
         readability, readingtype, sex, maturity_stage, superindividuals,
         length, weight)

attributes(ldist_surv)$updated <- updated

## Save

message("Saving length data...")

saveRDS(ldist_surv, file = "data/Length data for surveys.rds", compress = "xz")

############### #
# Catch data ####

message("Catch data")

mt.filter <- as.character(c(1:3, 9:12)) # Filter missiontypes

## Length distributions ####

message("Processing length data...")

aldist_land <- x %>%
  filter(
    commonname == "blåkveite",
    !is.na(length),
    missiontype %in% mt.filter,
    area %in% MainAreaFilter
    ) %>%
  filter(!is.na(latitudestart), !is.na(longitudestart)) %>%
  st_as_sf(coords = c("longitudestart", "latitudestart"), crs = 4326,
           remove = FALSE) %>%
  rownames_to_column("id") %>%
  st_join(strata.poly %>%
            mutate(population = paste0(geostrata.name, interval))) %>%
  filter(!duplicated(id)) %>%
  st_set_geometry(NULL) %>%
  mutate(
    sampling_type =
      recode(missiontype,
             !!!setNames(SamplingTypeList$name, SamplingTypeList$id)),
    gear = recode(gear, !!!setNames(gearList$gearcategory, gearList$code)),
    month = ifelse(is.na(stationstartdate),
                   lubridate::month(stationstopdate),
                   lubridate::month(stationstartdate)),
    sex = ifelse(sex > 2, NA, ifelse(sex == 1, "F", "M")),
    length = 100*length + 1e-6) %>% # addition to avoid a floating point issue
  replace_na(list(population = "Outside")) %>%
  filter(!is.na(month)) %>%
  rename(areacell = area, year = startyear, weight = individualweight,
         maturity_stage = maturationstage)

ldist_land <- aldist_land %>%
  select(gear, sampling_type, year, month, areacell, population, age,
         readability, readingtype, sex, maturity_stage, length, weight)


attributes(ldist_land)$updated <- updated

### Save

message("Saving length data...")

saveRDS(ldist_land, file = "data/Length data for catches.rds", compress = "xz")

## Age data ####

message("Age data...")

age_det_land <- aldist_land %>%
  dplyr::filter(!is.na(age)) %>%
  dplyr::filter(grepl("new", readingtype)) %>%
  mutate(cond = paste(sampling_type, platform, year, sep = "_")) %>%
  pull(cond) %>% table

aldist_land <- aldist_land %>%
  mutate(cond = paste(sampling_type, platform, year, sep = "_")) %>%
  filter(cond %in% names(age_det_land)) %>%
  replace_na(list(sex = "U")) %>%
  mutate(length_bin = cut(length, seq(0,115,5)))

aldist_land <- aldist_land %>%
  group_by(gear, sampling_type, year, month, areacell, population, platform,
           serialnumber, sex, length_bin) %>%
  reframe(total = n()) %>%
  left_join(
    aldist_land %>%
      filter(!is.na(age),
             grepl("new", readingtype)) %>%
      group_by(gear, sampling_type, year, platform, sex, age, length_bin) %>%
      count() %>%
      group_by(gear, sampling_type, year, platform, sex, length_bin) %>%
      mutate(aged = sum(n), pr = n/aged),
    multiple = "all",
    by = join_by(gear, sampling_type, year, platform, sex, length_bin),
    relationship = "many-to-many"
  ) %>%
  ungroup() %>%
  mutate(est_n = ifelse(is.na(pr), total, pr * total)) %>%
  dplyr::select(-aged, -pr, -n) %>%
  relocate(age, .after = "length_bin") %>%
  mutate(length = sapply(length_bin, split_fun), .before = length_bin)

attributes(aldist_land)$updated <- updated

message("Saving age data...")

saveRDS(aldist_land, file = "data/Age-length data for catches.rds", compress = "xz")



