## ---------------------------
##
## Script name: Run iterative reweighting
##
## ---------------------------

# Temp backend

if(Sys.info()["sysname"] == "Linux") {
  library(flexiblas)
  flexiblas_switch(flexiblas_load_backend("OPENBLAS-THREADS") )
}

## Set component weight to 1 unless it was 0 (assuming intentional removal)
if(set_weights) {
  tmb_param[grepl('weight$', tmb_param$switch) & tmb_param$value != 0,
            c("value", "lower", "upper", "optimise")] <-
    data.frame(value = 1, lower = NA, upper = NA, optimise = FALSE)
}

## Component grouping
component_grouping <-
  list(SI_Adults = c('EggaN_SI_female', 'EggaN_SI_male', 'EcoS_SI',
                     'RussianS_SI'), #"log_EggaS_SI", 
       SI_Juv = c('Juv_SI_1', 'Juv_SI_2'),
       TrawlNor = c('TrawlNor_ldist', 'TrawlNor_sexdist'),
       OtherNor = c('OtherNor_ldist', 'OtherNor_sexdist'), # , 'OtherNor_aldist'
       TrawlRus = c('TrawlRus_ldist', 'TrawlRus_sexdist'),
       # OtherRus = c('OtherRus_ldist', 'OtherRus_sexdist'),
       EcoS = c('EcoS_ldist', 'EcoS_sexdist'), # 'EcoS_aldist',
       EggaN = c('EggaN_aldist_female', 'EggaN_aldist_male', 'EggaN_ldist', 'EggaN_matp'),
       EggaS = c('EggaS_ldist', 'EggaS_matp'), # 'EggaS_aldist',
       RussianS = c('RussianS_ldist', 'RussianS_sexdist')
  )

## Remove components not listed in component grouping to avoid crashes later in g3_iterative
tmp_test <- setNames(lapply(unname(unlist(component_grouping)), function(k) {
  list(component_found = grep(k, tmb_param[grepl('weight$', tmb_param$switch),]$switch, value = T),
       non_zero_value = grep(k, tmb_param[grepl('weight$', tmb_param$switch) & tmb_param$value != 0,]$switch, value = T)
  )
}), unname(unlist(component_grouping)))

### Components not found (typos or removed from likelihood)
if(any(sapply(tmp_test, function(k) length(k$component_found) == 0))) {
  missing_components <- sapply(tmp_test, function(k) length(k$component_found) == 0)
  missing_components <- missing_components[missing_components]
  warning(paste(names(missing_components), collapse = ", "), " not found from tmb_param$switch. Removed from component grouping. If a typo, correct and rerun.")

  component_grouping <- lapply(component_grouping, function(k) {
    k[!k %in% names(missing_components)]
  })

  tmp_test <- setNames(lapply(unname(unlist(component_grouping)), function(k) {
    list(component_found = grep(k, tmb_param[grepl('weight$', tmb_param$switch),]$switch, value = T),
         non_zero_value = grep(k, tmb_param[grepl('weight$', tmb_param$switch) & tmb_param$value != 0,]$switch, value = T)
    )
  }), unname(unlist(component_grouping)))

  ### Components with 0 weight
} else if (any(sapply(tmp_test, function(k) length(k$non_zero) == 0))) {
  missing_components <- sapply(tmp_test, function(k) length(k$non_zero) == 0)
  missing_components <- missing_components[missing_components]
  warning(paste(names(missing_components), collapse = ", "), " found from tmb_param$switch, but has 0 weight. Removed from component grouping. Make parameter weight != 0 if you want to keep it in g3_iterative.")

  component_grouping <- lapply(component_grouping, function(k) {
    k[!k %in% names(missing_components)]
  })
}

## Run iterative

time_iter_start <- Sys.time()
message("Iteration started ", time_iter_start)

iter_param <- g3_iterative(
  gd = base_dir,
  wgts = "iterative_reweighting",
  model = tmb_model,
  params.in = tmb_param,
  grouping = component_grouping,
  use_parscale = TRUE,
  control = list(maxit = maxit),
  cv_floor = 4e-4, # Gives maximum weight of 1/cv_floor for survey indices
  shortcut = FALSE,
  mc.cores = ncores
)

time_iter_end <- Sys.time()
time_iter <- round(as.numeric(time_iter_end - time_iter_start, units = "mins"), 1)
message("Iteration finished ", time_iter_end, " after ", time_iter, " min")

if(!file.exists(file.path(base_dir, "session/run_times.txt"))) {
  info_file <- file(file.path(base_dir, "session/run_times.txt"))
  close(info_file)
}

cat(
  c("Iteration:\n",
    "   started ", as.character(time_iter_start), "\n",
    "   finished ", as.character(time_iter_end), "\n",
    "   time ", time_iter, " min", "\n\n"),
  file = file.path(base_dir, "session/run_times.txt"), sep = "", append = TRUE)

### Save the model parameters

write.csv(as.data.frame(iter_param),
          file = file.path(base_dir, "iterative_reweighting/Iterated TMB parameters.csv"))
save(iter_param,
     file = file.path(base_dir, "iterative_reweighting/Iterated TMB parameters.rda"), compress = "xz")

### Plots

iter_fit <- g3_fit(model, iter_param)
save(iter_fit,
     file = file.path(base_dir, "iterative_reweighting/Iterated TMB model fit.rda"), compress = "xz")

# gadget_plots(iter_fit, file.path(base_dir, "figures"))

if(plot_html) {
  tmppath <- file.path(getwd(), base_dir, "figures")
  make_html(iter_fit, path = tmppath, file_name = "model_output_figures_iter.html", template = "nea_ghl")
  rm(tmppath)
}
