## ---------------------------
##
## Script name: Optimize the gadget3 model for NEA Greenland halibut
##
## Purpose of script: Master script to run the model. Source the entire script or run from top to a desired line (Ctrl+Alt+B on Rstudio)
##
## Authors:
## Mikko Vihtakari and Daniel Howell // Institute of Marine Research, Norway
## Will Butler and Bjarki Elvarsson // Marine and Freshwater Research Institute, Iceland
## Email: mikko.vihtakari@hi.no
##
## ---------------------------

##################################################
## Instructions to run the script in terminal ####
## You can only the model in one session / folder (computer) simultaneously
## To run in terminal through screen on Eucleia, do:
## cd to the desired ghl-gadget3 folder and edit _RUN_THE_MODEL.R using Vim (vim _RUN_THE_MODEL.R)
# screen -L -Logfile gadgetrun.log -S gadgetrun
# output from file.path(R.home("bin"), "R") or locate -b R | fgrep -w R/bin:
# /opt/apps/gcc-toolset-12/R/4.3.3/lib64/R/bin/R
# setwd("nea-ghl-gadget")
# source("_RUN_THE_MODEL.R", echo = TRUE)
## Ctrl-a-d # to detach (close and let the process run)
# screen -r gadgetrun # to reattach (look into the process)

## You can also automize the process above, but plotting html does not work for some reason. Switch plot_html to FALSE and produce the plots later.
# screen -LS gadgetrun bash -c '/software/R-4.2.1/bin/Rscript --verbose _RUN_THE_MODEL.R'

## Once done, you can kill inactive screen sessions (think before you do this):
# pkill screen
## Remember not to start another run in Rstudio server while the screen session is running
## Also running two screen sessions using the same ghl-gadget-data is not possible because duckdb does not support multiple connections. Kill the old screen sessions for the same folder before starting a new one.

## Alternatively you can run it using nohup (this option does not seem to work)
# cd to the desired ghl-gadget3 folder.
# nohup /software/R-4.2.1/bin/R -e 'source("_RUN_THE_MODEL.R", echo=TRUE, chdir = TRUE)' &

##################################
# Source the run first script ####

rm(list = ls())

source("0 run first.R")

# Model settings ####

base_dir <- "model_files" # All files and output of the currently run model will be placed in a folder with this name

## Run options ####
run_optim <- TRUE # Whether to run optimization of parameters (the normal routine)
run_iterative <- FALSE # Whether to run iterative reweighing (takes 3-10 hours)
run_jitter <- FALSE # Run jittered optimization. Repeats g3_optim multiple times with slightly different starting values.
run_retro <- FALSE # Run retrospective analysis?
run_leaveout <- TRUE # Runs a leave-out analysis with likelihood components to leave out (using 0 weight) specified in leaveout_re

run_bootstrap <- FALSE # Not implemented yet

## General options ####
reset_model <- TRUE # Change to TRUE to reset the model (delete all model files). ONLY do this if you really want to DELETE the existing model
reload_data <- FALSE # Set this to true to reload data from sources.
plot_html <- TRUE # Whether html model summary should be plotted. In most cases you want this TRUE unless you work on a server that doesn't have pandoc installed.
plot_retro_html <- FALSE # Whether all 6 retro fit objects should be plotted to separate html files. Useful for debugging but takes 5*15 MB more space

## Parameter and likelihood component options ####
set_weights <- TRUE # Whether to set manual weights for likelihood components from previous iterative reweighting. The weights are defined in 6 initial parameters.R
force_bound_params <- TRUE # Whether parameters should be forced to their bounds.
previous_model_params_as_initial <- TRUE # Whether to use parameters optimized parameters as initial values for tmb_params. Speeds up the optimization, but also sets the model to a certain likelihood-scape.

## Optimisation settings (adjust here and they'll change everywhere)
ncores <- 10 # Number of cores to use for parallel operations. The number of available computer cores is often not a limiting factor when running gadget, but memory is because the entire model is copied to RAM multiple times. Use max ncores = 10 for Eucleia, and ncores = 1 or a few for other servers.
maxit <- 3000 # Maximum number of iterations during an optimization run.

## Fleet selectivity

selectivity <- "s" # Fleet selectivity: "s" uses sigmoid-shaped (g3_suitability_exponentiall50) selectivity for all fleets; "dome" dome-shaped (g3_suitability_andersen) selectivity for EcoS_survey, WinterS_survey, Internat_fishery, OtherNor_fishery, OtherRus_fishery, TrawlNor_fishery, and TrawlRus_fishery.

## Optimization mode (param_opt_mode), options:
# (1) parameters are bounded internally (ie using the bounded function) works with 'BFGS' optim method
# (2) parameters are bounded externally so the optim function must use a box-constrained optimization method
# (3) global search, all parameters unbounded, unconstrained optimization can be used (eg 'BFGS')

## How should the initial abundance be setup?
## Options:
## 0 - population is initialized at equilibrium
## 1 - parameter for each age group (across stocks)
## 2 - parameter for each age group of each stock
## 4 - two parameters: init_F and init.scalar. Population initiated 50/50 males/females, considers M and init_F for sexes.

setup_options <- list(param_opt_mode = 1,
                      initial_abund_mode = 4)

## Whether or not to bound parameters internally
setup_options$bound_params <- ifelse(setup_options$param_opt_mode == 1, TRUE, FALSE)

# Compile the model ####

### Model and stock parameters, setting up areas, time steps, etc. ####

source("1 settings.R")

### Get data ####

source("2 data.R")

### Setup the stocks ####

source("3 stocks.R")

### Add fleets and landings data ####

source("4 fleets.R")

### Setup likelihood components ####

source("5 likelihood.R")

### Formulate R based model and define initial parameters ####

source("6 parameters.R")

### Turn off likelihood components
# tmb_param <- tmb_param %>% g3_init_guess('aldist', 0, NA, NA, 0)
# tmb_param$value$cdist_sumofsquares_EggaN_aldist_female_weight <- 1
# tmb_param <- tmb_param %>%
#   g3_init_guess('aldist', 0, NA, NA, 0)
#   g3_init_guess('RussianSurvey_SI', 0, NA, NA, 0) %>%
#   g3_init_guess('Juv_SI_3', 0, NA, NA, 0)
#   g3_init_guess('EcoS', 0, NA, NA, 0) %>%
#   g3_init_guess('EggaS', 0, NA, NA, 0) %>%
#   g3_init_guess('aldist', 0, NA, NA, 0) %>%
#   g3_init_guess('sexdist', 0, NA, NA, 0)

## Fit the initial parameters to the model, print the likelihood score and make plots which will be overwritten by optimized parameter plots later.

# result <- model(tmb_param$value)
# result[[1]]

# init_fit <- gadgetutils::g3_fit(model,tmb_param)
# tmppath <- file.path(getwd(), base_dir, "figures")
# make_html(init_fit, path = tmppath, file_name = "model_output_figures_init.html")
# rm(tmppath)

# png(file.path(base_dir, "figures/Initial_annual_plot.png"), width = pagewidth, height = pagewidth, units = "mm", res = 300)
# plot_annual(fit_init)
# dev.off()

# List all available reports
# print(names(attributes(result)))

## Compile the TMB-based model

# model_tmb <- g3_tmb_adfun(tmb_model, tmb_param)
#
# save(model_tmb, file = file.path(base_dir, "data/TMB model.rda"), compress = "xz")

## Copy the R scripts used to compile the model
if(reset_model) {
  file.copy(dir(pattern = "\\.R$"), file.path(getwd(), base_dir, "scripts"))
}

###############################################
## Iterative reweighting and optimization  ####
## Running this part takes a long time (3-10 hours on a server)  ###
if(run_iterative & reset_model) {
  source("7 iterative reweighting.R")
} else if(run_iterative & !reset_model & dir.exists(file.path(base_dir, "iterative_reweighting"))) {
  load(file.path(base_dir, "iterative_reweighting/Iterated TMB parameters.rda"))
  load(file.path(base_dir, "iterative_reweighting/Iterated TMB model fit.rda"))
}

#################################
## Optimize model parameters ####
if(run_optim & reset_model) {
  source("8 optimise.R")
} else if(run_optim & !reset_model & dir.exists(file.path(base_dir, "optim"))) {
  load(file.path(base_dir, "optim/Optimized TMB parameters.rda"))
  load(file.path(base_dir, "optim/Optimized TMB model fit.rda"))
}

###############################
## Jitter model parameters ####
## Takes njits(10)/ncores times optimisation time
if(run_jitter & reset_model) {
  source("9 jitter.R")
}

#########################
## Retroscpetive run ####
if(run_retro & reset_model) {
  source("10 retro.R")
}

###########################
## Leave out analysis  ####
## Takes length(leaveout_re)/ncores times optimisation time ###
if(run_leaveout) {
  source("11 leave-out.R")
}

## Save workspace

savehistory(file = file.path(base_dir, "session/.Rhistory"))
save.image(file = file.path(base_dir, "session/gadget_workspace.RData"), compress = "xz")
message("Script finished ", Sys.time(), ". Saved to ", base_dir)

if(exists("optim_fit") & exists("time_optim")) {
  message("Optimisation", ifelse(attributes(optim_fit)$summary$convergence, " converged.", " did not converge."),
          " It took ", attributes(optim_fit)$summary$gd_calls,
          " iterations and ", round(time_optim/60, 1), " hours to run.")
}

## When running Ctrl + Alt + B, run until here. The options should take care what gets evaluated.
